/* Terraform - (C) 1997-1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#ifndef _GUI_BUFFERED_DRAWING_AREA_H
#define _GUI_BUFFERED_DRAWING_AREA_H


#include <gtk--.h>
#include <gtk/gtk.h>


/*
 *  GuiBufferedDrawingArea: provide an offscreen drawing area which 
 *	is refreshes/repaints using an atomic copy operation from the 
 *      offscreen pixmap to the on screen window. 
 */
class GuiBufferedDrawingArea : public Gtk_DrawingArea
	{
	public:
				GuiBufferedDrawingArea ();
				~GuiBufferedDrawingArea ();
		void		clear (GdkColor *col=NULL);
		void		sync (GdkRectangle *update_rect=NULL);
		void		setSync (int syncInterval);
		void		setColors (GdkColor *fg, GdkColor *bg=NULL);
		void 		drawPoint (int x, int y);
		void 		drawLine (int x1, int y1, int x2, int y2);
		void 		drawRectangle (int x1, int y1, int width, int height, bool filled=TRUE);
		void 		drawPolygon (GdkPoint *points, gint npoints, bool filled=TRUE);
		void 		drawBorderedPolygon (GdkPoint *points, gint npoints, GdkColor *borderColor, GdkColor *polygonColor);
		void		drawText (int x1, int y1, char *textData, GdkFont *textFont);
		void		drawPixmap (GdkPixmap *pixmap, int xsrc, int ysrc, 
					int xdest, int ydest, int width, int height);
		void		setUsize (int x, int y);

	private: 
		int 		handleConfigureEvent (GdkEventConfigure *event);
		int		handleExposeEvent (GdkEventExpose *event);
		int 		handleButtonPressEvent (GdkEventButton *event);
		int 		handleMotionNotifyEvent (GdkEventMotion *event);

		unsigned int	d_syncInterval,	// interval at which we sync
				d_noSyncCount,
				d_sMaxx, d_sMinx, 	// area which is 
				d_sMaxy, d_sMiny; 	// to be synced
		GdkPixmap 	*d_pixmap; 	// pixmap buffer for drawing area 
		GdkGC		*d_GC; 		// graphics content
	};



#endif // _GUI_BUFFERED_DRAWING_AREA_H
