/* Terraform - (C) 1997-1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "GuiColormap.h"
#include "GlobalTrace.h"
#include "GlobalSanityCheck.h"


#define LOAD		0
#define SAVE		1
#define BANDSIZE	15



/*
 *  constructor: initialize everything 
 */
GuiColormap::GuiColormap (Gtk_Widget *parent, int maxcolors)
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ GuiColormap\n");

	parent->realize ();
	d_colormap = parent->get_colormap ();
	d_ncolors = 0;
	d_maxcolors = maxcolors;

	SanityCheck::bailout ((!this->allocNamedColor (&d_black, "black")), 
				"can't alloc color black", "FileIO::open");
	SanityCheck::bailout ((!this->allocNamedColor (&d_white, "white")), 
				"can't alloc color white", "FileIO::open");

	d_colors = new GdkColor[maxcolors];
}


/*
 *  destructor: free allocated memory
 */
GuiColormap::~GuiColormap ()
{
	if (d_colors)
		delete [] d_colors;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- GuiColormap\n");
}


/*
 *  getColor: get the color at the specified index
 */
GdkColor *GuiColormap::getColor (int index)
{
	if (index < d_ncolors && index >= 0)
		return (&(d_colors[index]));
	else
		{
		g_print ("Warning: GuiColormap index (%d) out of range (0, %d)\n", index, d_ncolors-1);
		return NULL;
		}
}


/*
 *  numColors: return the number of colors in the colormap
 */
int GuiColormap::numColors ()
{
	return (d_ncolors);
}


/*
 *  addColor: add a color to the colormap
 */
bool GuiColormap::addColor (GdkColor *col, int r, int g, int b) 
{
	if (allocColor (col, r, g, b))
		{
		d_colors[d_ncolors++] = *col;
		return (d_ncolors);
		}
	else
		return (-1);
}


/*
 *  addColor: add a color to the colormap
 */
bool GuiColormap::addColor (GdkColor	*col)
{
	if (allocColor (col))
		{
		d_colors[d_ncolors++] = *col;
		return (d_ncolors);
		}
	else
		return (-1);
}


/*
 *  addNamedColor: add a named color to the colormap
 */
bool GuiColormap::addNamedColor (GdkColor	*col, 
				char		*colname)
{
	if (allocNamedColor (col, colname))
		{
		d_colors[d_ncolors++] = *col;
		return (d_ncolors);
		}
	else
		return (-1);
}


/*
 *  addNamedColor: allocate a color 
 */
bool GuiColormap::allocColor (GdkColor *col, int r, int g, int b) 
{
	col->red=r;
	col->green=g;
	col->blue=b;
	if (gdk_color_alloc (d_colormap, col))
		return (TRUE);
	else
		return (FALSE);
}


/*
 *  addNamedColor: allocate a color 
 */
bool GuiColormap::allocColor ( 	GdkColor	*col)
{
	if (gdk_color_alloc (d_colormap, col))
		return (TRUE);
	else
		return (FALSE);
}


/*
 *  addNamedColor: allocate a named color 
 */
bool GuiColormap::allocNamedColor (	GdkColor	*col, 
					char		*colname)
{
	if (	gdk_color_parse (colname, col) && 
		gdk_color_alloc (d_colormap, col))
		return (TRUE);
	else 
		{
		cerr << "Unable to allocate color " << colname << " ... Exiting\n";
		//destroy_callback ();
		return (FALSE);
		}
}


