/* Terraform - (C) 1997-1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include <string.h>
#include <stdlib.h>
#include "GuiDialogYN.h"
#include "GlobalTrace.h"



/*
 *  constructor: initialize everything 
 */
GuiDialogYN::GuiDialogYN(char *winTitle, char *frameTitle)
	      : Gtk_Dialog (),
		d_btnYes ("Yes"),
		d_btnNo ("No")
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ GuiDialogYN\n");

	if (winTitle)
		d_winTitle = strdup (winTitle);
	else
		d_winTitle = strdup ("GuiDialogYN Window");

	if (frameTitle)
		d_frameTitle = strdup (frameTitle);
	else
		d_frameTitle = NULL;

	d_baseFrame = NULL;
	this->set_position (GTK_WIN_POS_MOUSE);		
}


/*
 *  destructor: clean up, free allocated memory
 */
GuiDialogYN::~GuiDialogYN()
{
	if (d_winTitle) free (d_winTitle);	// c-type alloc from strdup
	if (d_frameTitle) free (d_frameTitle);	// c-type alloc from strdup
	if (d_baseFrame) delete d_baseFrame;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- GuiDialogYN\n");
}


/*
 *  buildDialogWindow: call the proper functions to setup the dialog
 */
void GuiDialogYN::buildDialogWindow ()
{
	this->set_usize (300, 120);
	this->set_title (d_winTitle);

	this->get_vbox()->set_border_width (2);

	setupVBox ();
	fillVBox ();
	fillActionArea ();

	this->show ();
}


/*
 *  setupVBox: setup the basic frame of the vbox if necessary 
 */
void GuiDialogYN::setupVBox ()
{
	if (d_frameTitle)
		{
		d_baseFrame = new Gtk_Frame (d_frameTitle);
		d_baseFrame->set_shadow_type (GTK_SHADOW_ETCHED_IN);
		d_baseFrame->set_border_width (6);
		this->get_vbox()->pack_start (*d_baseFrame, TRUE, TRUE, 0);
		d_baseFrame->show ();	
		}
	this->get_vbox()->show ();
}


/*
 *  fillVBox: fill the dialog's VBox 
 */
void GuiDialogYN::fillVBox ()
{
}


/*
 *  fillActionArea: fill the dialog's action area 
 */
void GuiDialogYN::fillActionArea ()
{
	connect_to_method (d_btnYes.clicked, this, &buttonCallbackYes);
	this->get_action_area()->pack_start (d_btnYes, TRUE, TRUE, 0);
	d_btnYes.show ();

	connect_to_method (d_btnNo.clicked, this, &buttonCallbackNo);
	this->get_action_area()->pack_start (d_btnNo, TRUE, TRUE, 0);
	d_btnNo.show ();
}


void GuiDialogYN::buttonCallbackYes ()
{
	cout << "GuiDialogYN: Default Callback Yes\n";
}


void GuiDialogYN::buttonCallbackNo ()
{
	cout << "GuiDialogYN: Default Callback No\n";
	this->hide ();
}


gint GuiDialogYN::delete_event_impl(GdkEventAny *)
{
	this->hide ();
	return (0);
}

/*
int main (int argc, char *argv[])
{
	Gtk_Main	gtkMain (&argc, &argv);
	GuiDialogYN	*testDialog;

	testDialog = new GuiDialogYN ("GuiDialogYN", "Test Frame");
	testDialog->buildDialogWindow ();
	testDialog->show ();
	gtkMain.run ();
}
*/
