/* Terraform - (C) 1997-1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#ifndef _HEIGHT_FIELD_DRAW_H
#define _HEIGHT_FIELD_DRAW_H

#include "HeightField.h"
#include "HeightFieldModel.h"
#include "TFWindow.h"
#include "GuiColormap.h"
#include "MathTrig.h" 		// PT2 and PT3 defs


#define MENU_BAR_OFFSET	25
#define HFWIN_MINX	 	200
#define HFWIN_MINY	 	50+MENU_BAR_OFFSET
#define HFWIN_SIZEX(thf) 	(thf->Width()<HFWIN_MINX ? HFWIN_MINX : thf->Width())
#define HFWIN_SIZEY(thf) 	thf->Height()+MENU_BAR_OFFSET

#define MAXCOL			225	// 15 x 15
#define BANDSIZE		15

#define PLANE 			0
#define WIRE   			1                	// wireframe 
#define HEIGHT		 	2			// wireframe + height
#define SLOPE  			3
#define LIGHT  			4
#define NDM			PLANE			// Number of Draw Modes

#define LOAD			0
#define SAVE			1

#define         PTYPE                   float

#define D 	double

#define MIN_SCREEN_SIZE	10
#define MAX_SCREEN_SIZE	800
#define MIN_SCALE_FACTOR 0.25
#define MAX_SCALE_FACTOR 10


/*
 *  HeightFieldDraw: draws a height field in 2 or 3 D. 
 * 	Depends on being declared a friend to HeightField
 */
class HeightFieldDraw
	{
	public:
				HeightFieldDraw(HeightField *HF,TFWindow *tfWin);
				HeightFieldDraw(HeightField *HF,GuiBufferedDrawingArea *drawArea);
				~HeightFieldDraw ();
		int		setColormap (GuiColormap *colorMap, int nbands=0, int bandsize=0, int blandoffset=0);
		int		setMode (char *s);
		int		setScale (float s, int *x=NULL, int *y=NULL);
		int		alterScale (float f, int *x=NULL, int *y=NULL);
		void		setSync (int syncRate);
		void		setFastWire (bool fastWire);
		void 		setEyePos (double _x_, double _y_, double _z_);
		void 		setLightPos (double _x_, double _y_, double _z_);
		void 		setYScale (double yscale);
		void 		setSeaFill (int sfill);
		void 		setResolution (int i);
		void		setFastOptions (bool halfWireRes, bool halfYscale,
						bool doRotZ);
		int		draw (bool changedHF=TRUE);
		void		resetView ();
		void		clear ();
		void		printSettings ();
		void		help ();
		int		rotateFastView (float a, float b, float c);
		int		getMode () 		{return d_shader;}
		GuiColormap	*getColormap () 	{return d_cMap;}

		bool		d_fastDoRotZ;	// accessible for TFGui.cc
	private:
		int		tri3d (D x0, D z0,D x1,D z1,D y0,D y1,D y2,D y3, int offset); 
		int		drawhf2d ();
		int		drawhf3dFast ();
		int		drawhf3d ();
		PTYPE		hf_abs (PTYPE n);

		HeightField	*d_HF;
		HeightFieldModel*d_HFM;
		GuiColormap	*d_cMap;
		TFWindow	*d_tfWin;
		GuiBufferedDrawingArea *d_drawArea;
		PT3 		d_light;  			// position of light source 
		PT3 		d_eye;    			// position of viewer 
		D		d_zcenter, 			
				d_zsf, d_ysf, 			// z & y coordinate scale factors
				d_ycorr, d_zeps, 
				d_bright, d_eyeinc,
				d_slMin, d_slScale,
				d_scale;
		PTYPE		d_nrange, d_prange,
				d_fastWireResAdj,
				d_fastYscaleAdj;
		int		d_resolution, 				// resolution 
				d_wireResolution,
				d_ncolors, 			// max number of colors
				d_nbands,
				d_bandsize, 
				d_xgsize, d_ygsize, 		// size of display window
				d_bandLandOffset,		// land offset in X_Color table in bands and in colors
				d_yoff, d_maxcon, 
				d_cmin, d_cmax, 
				d_erasewin, 
				d_tile, d_shader, 
				d_sxmid, d_symid;
		bool		d_sea, 
				d_fastWire; 
	};

#endif // _HEIGHT_FIELD_DRAW_H
