/* Terraform - (C) 1997-1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#ifndef _HEIGHT_FIELD_GEN_SUBDIV_H
#define _HEIGHT_FIELD_GEN_SUBDIV_H


#include "HeightField.h"
#include "MathGauss.h"


#define MENU_GEN_DIAMOND	"Diamond Square"
#define MENU_GEN_MIDPOINT	"Midpoint Subdivision"
#define MENU_GEN_OFFSET		"Offset Square"
#define MENU_GEN_RECURSIVE	"Recursive Subdivision"


/*
 *  HeightFieldGenSubdiv: generate a height field by subdivision. 
 * 	Depends on being declared a friend to HeightField
 */
class HeightFieldGenSubdiv
	{
	public:
				HeightFieldGenSubdiv (HeightField *HF);
				~HeightFieldGenSubdiv();
		void		initHF (int n);
		void		setRequestedSize ();
		int		setPreferredSize (int n);
		float		avgyvals (int i, int j, int strut, int dim);
		float		avgyvals2 (int i, int j, int strut, int dim);
		int		generate (int mode, int n, float scaleFactor, int seed);
		int		generateDiamond (int n, float scaleFactor, int seed);
		int		generateOffset (int n, float scaleFactor, int seed);
		int		generateMidpoint (int n, float scaleFactor, int seed);
		int		generateRecursive (int n, float scaleFactor, int seed);

	private:
		void		processRSquare (int x1, int y1, int x2, int y2, 
					float maxdelta);

		HeightField	*d_HF;
		MathGauss	*d_mGauss;
		int		d_prefSize,		// preferred (2^n) size
				d_reqSize;		// requested size 
		float		d_RscaleFactor;		// recursive scale factor
	};


#endif // _HEIGHT_FIELD_GEN_SUBDIV_H

