/* Terraform - (C) 1997-1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#ifndef _HEIGHT_FIELD_OPS_H
#define _HEIGHT_FIELD_OPS_H


#include "HeightField.h"
#include "HeightFieldDraw.h"


/*
 *  HeightFieldOps: class for general height field utilities/transformations.
 * 	Depends on being declared a friend to HeightField
 */
class HeightFieldOps
	{
	public:
				HeightFieldOps (HeightField *HF);
				~HeightFieldOps();

		int		upslopeSFD (int x, int y, PTYPE *cache);
		float		upslopeMFD (int x, int y, PTYPE *cache);
		HeightField	*flowmap (bool doSFD, bool ignoreSealevel=TRUE, 
					float maxElvErode=1.0);
		int		erode (int nTurns, int maxFlowAge, 
				       float maxElvErode, char *fName, 
				       bool trimLocalPeaks, bool doSFD, 
				       bool ignoreSealevel, HeightFieldDraw *HFD=NULL);
		int		hfDouble (double f, double f2);
		int		hfHalf ();
		HeightField	*hfResize (int newx=100, int newy=100);
		int		rotate90 (int degrees);
		int		roughSmooth (bool roughen, bool bigGrid, float factor);
		int		transform (float factor, int adjust_sea_level=TRUE);
		int		invert (int adjust_sea_level=FALSE);
		int		fold (int falloff);
		int		linearScale (float centerx, float centery, float scaleFactor, float mindist=0.0, float maxdist=1.0, float smoothFactor=0.0, bool invertScale=FALSE, bool scaleFullDist=TRUE);
		int		generate (int x=200, int y=200);
		int		mirror (int type);
		int		fill (PTYPE elv, float factor, bool keepMin);
		int		placePOVObject (char *outfile, char *object, 
					int hfsx, int hfsy, int hfsz, 
					PTYPE floor, PTYPE ceil, 
					float density, float pvar, 
					int osx, int osy, int osz,
					PTYPE osxvar, PTYPE osyvar, PTYPE oszvar, 
					int seed=0);

	private:
		static int	hf_sort (const void *v1, const void *v2);

		long		d_cHit, d_cMiss; // cache hit/miss counts
		HeightField	*d_HF;
	};


#endif // _HEIGHT_FIELD_OPS_H 

