/* Terraform - (C) 1997-1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#ifndef _MATH_TRIG_H
#define _MATH_TRIG_H


// 2D point
typedef struct Point2 {
	double x, y;
	} PT2;

// 2D point with depth marker
typedef struct Point2depth {
	double x, y, depth;
	} PTD2;

// 3D point
typedef struct Point3 {
	double x, y, z;
	} PT3;


// Triangle defined by three vertices
typedef struct TriByVertices {
	int 		v1, v2, v3;	// Indicies for triangle vertices
	unsigned int	cIndex;		// colormap index 
	PT3		normal;		// Surface normal vector
	bool 		visible;	// Should polygon be rendered?
	} TriVert;


// 3d object arranged as triangles
typedef struct TriModel3D {
	int 		nTri;		// Number of triangles in the model
	struct PT3	*data;		// elevation matrix
	struct TriVert 	*tri;		// Pointer to array of faces
	int 		xsize, ysize;
	} TriMod3D;


// 2d object arranged as triangles
typedef struct TriModel2d {
	int 		nTri;		// Number of triangles in the model
	struct PTD2	*data;		// List of model vertices
	struct TriVert  *tri;		// Pointer to array of faces
	int 		xsize, ysize;
	} TriMod2D;


/*
 *  MathTrig: general trig-related functions 
 */
class MathTrig
	{
	public: 
		static double		distance (int x1, int y1, int x2, int y2);
		static double		distance (PT2 p1, PT2 p2);
		static void		normalVector (TriMod3D model, TriVert *m);
	};

#endif // _MATH_TRIG_H
