/* Terraform - (C) 1997-1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite GuiDialogOAC::s_HBOffGuiDialogOAC::s_HBOff0,
 * Boston, MA 02111-1GuiDialogOAC::s_HBOff07, USA.
 */


#include "TFDialogErode.h"
#include "GlobalSanityCheck.h"
#include "GlobalTrace.h"


/*
 *  constructor: initialize all data members 
 */
TFDialogErode::TFDialogErode (HeightField *HF, HeightFieldDraw *HFD)
	        : GuiDialogOAC ("Terraform Erode Dialog", "Erosion Parameters"),
		  d_vBoxMain (FALSE, 5),
		  d_vBoxList1 (TRUE, 5),
		  d_vBoxList2 (TRUE, 5),
		  d_hBoxHS1 (TRUE, 5),
		  d_hBoxHS2 (TRUE, 5),
		  d_hBoxHS3 (TRUE, 5),
		  d_hBoxSave (TRUE, 5),
		  d_hBoxTrim (TRUE, 5),
		  d_frame1 ("Erosion Controls"),
		  d_frame2 ("Flowmap Controls"),
		  d_adjNTurns (5000, 1000, 25000, 0.01),
		  d_adjMaxElvErode (1.0, 0.1, 1.0, 1),
		  d_adjMaxFlowAge (250, 0, 1000, 1),
		  d_hsNTurns (d_adjNTurns),
		  d_hsMaxElvErode (d_adjMaxElvErode),
		  d_hsMaxFlowAge (d_adjMaxFlowAge),
		  d_lblHS1 ("Number of Iterations"),
		  d_lblHS2 ("Maximum flowmap age"),
		  d_lblHS3 ("Elevation Threshold"),
		  d_lblSave ("Save at max flowmap age"),
		  d_lblTrim ("Trim local peaks"),
		  d_hSep ()
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFDialogErode\n");

	d_HFO = NULL;
	setHFobjs (HF, HFD);
	d_RB1 = d_RB2 = NULL;
	d_CBtrim = d_CBsave = d_CBseaLevel = NULL;
	d_group = NULL;

	d_frame1.set_shadow_type (GTK_SHADOW_ETCHED_IN);
	d_frame1.set_border_width (5);
	d_frame2.set_shadow_type (GTK_SHADOW_ETCHED_IN);
	d_frame2.set_border_width (5);

	buildDialogWindow ();
}


/*
 *  destructor: clean up 
 */
TFDialogErode::~TFDialogErode ()
{
	if (d_HFO) delete d_HFO;
	if (d_RB1) delete d_RB1;
	if (d_RB2) delete d_RB2;
	if (d_CBseaLevel) delete d_CBseaLevel;
	if (d_CBsave) delete d_CBsave;
	if (d_CBtrim) delete d_CBtrim;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFDialogErode\n");
}


/*
 *  setHBobjs: set the HB objects we operate on 
 */
void TFDialogErode::setHFobjs (HeightField *HF, HeightFieldDraw *HFD)
{
	SanityCheck::bailout ((!HF), "HF==NULL", "TFDialogErode::setHFobjs");
	SanityCheck::bailout ((!HFD), "HFD==NULL", "TFDialogErode::setHFobjs");

	if (HF==d_HF && HFD==d_HFD)
		return;

	if (d_HFO) delete d_HFO;

	d_HF = HF;
	d_HFD = HFD;
	d_HFO = new HeightFieldOps (d_HF);
}


/*
 *  buildErodeFrame: build the frame for the erode parameters
 */
void TFDialogErode::buildErodeFrame ()
{
	d_hBoxHS1.pack_start (d_lblHS1, TRUE, TRUE, s_HBOff);
	d_lblHS1.show ();
	d_hBoxHS1.pack_start (d_hsNTurns, TRUE, TRUE, s_HBOff);
	d_hsNTurns.set_digits (0);
	d_hsNTurns.show();
	d_vBoxList1.pack_start (d_hBoxHS1, TRUE, TRUE, s_VBOff);
	d_hBoxHS1.show ();

	d_hBoxHS2.pack_start (d_lblHS2, TRUE, TRUE, s_HBOff);
	d_lblHS2.show ();
	d_hBoxHS2.pack_start (d_hsMaxFlowAge, TRUE, TRUE, s_HBOff);
	d_hsMaxFlowAge.set_digits (0);
	d_hsMaxFlowAge.show();
	d_vBoxList1.pack_start (d_hBoxHS2, TRUE, TRUE, s_VBOff);
	d_hBoxHS2.show ();

	d_hBoxHS3.pack_start (d_lblHS3, TRUE, TRUE, s_HBOff);
	d_lblHS3.show ();
	d_hBoxHS3.pack_start (d_hsMaxElvErode, TRUE, TRUE, s_HBOff);
	d_hsMaxElvErode.set_digits (2);
	d_hsMaxElvErode.show();
	d_vBoxList1.pack_start (d_hBoxHS3, TRUE, TRUE, s_VBOff);
	d_hBoxHS3.show ();

	d_hBoxTrim.pack_start (d_lblTrim, TRUE, TRUE, s_HBOff);
	d_lblTrim.show ();
	d_CBtrim = new Gtk_CheckButton ();
	d_CBtrim->set_active (TRUE);
	d_hBoxTrim.pack_start (*d_CBtrim, TRUE, TRUE, s_HBOff);
	d_CBtrim->show ();
	d_vBoxList1.pack_start (d_hBoxTrim, TRUE, TRUE, s_VBOff);
	d_hBoxTrim.show ();

	d_CBsave = new Gtk_CheckButton ("Save when refreshing Flowmap");
	d_CBsave->set_active (FALSE);
	d_hBoxSave.pack_start (*d_CBsave, TRUE, TRUE, s_HBOff);
	d_CBsave->show ();
	connect_to_method (d_CBsave->clicked, this, &checkboxCallback);
	d_hBoxSave.pack_start (d_entFileName, TRUE, TRUE, s_HBOff);
	d_entFileName.set_text ("erode.tga");
	d_entFileName.set_sensitive (FALSE);
	d_entFileName.show ();
	d_vBoxList1.pack_start (d_hBoxSave, TRUE, TRUE, s_VBOff);
	d_hBoxSave.show ();

	d_frame1.add (d_vBoxList1);
	d_vBoxList1.show ();

	d_vBoxMain.pack_start (d_frame1, FALSE, TRUE, s_VBOff);
	d_frame1.show ();
}


/*
 *  buildFlowmapFrame: build the frame for the flowmap parameters
 */
void TFDialogErode::buildFlowmapFrame ()
{
	d_RB1 = new Gtk_RadioButton ((GSList *)NULL, "Single Flow Direction");
	d_RB1->set_active (TRUE);
	d_vBoxList2.pack_start (*d_RB1, TRUE, TRUE, s_VBOff);
	d_RB1->show ();

	d_group = d_RB1->group ();
	d_RB2 = new Gtk_RadioButton (d_group, "Multiple Flow Direction");
	d_vBoxList2.pack_start (*d_RB2, TRUE, TRUE, s_VBOff);
	d_RB2->show ();

	d_vBoxList2.pack_start (d_hSep, FALSE, FALSE, 0);
	d_hSep.show ();

	d_CBseaLevel = new Gtk_CheckButton ("Observe Sealevel");
	d_CBseaLevel->set_active (FALSE);
	d_vBoxList2.pack_start (*d_CBseaLevel, TRUE, TRUE, s_VBOff);
	d_CBseaLevel->show ();

	d_frame2.add (d_vBoxList2);
	d_vBoxList2.show ();

	d_vBoxMain.pack_start (d_frame2, FALSE, FALSE, s_VBOff);
	d_frame2.show ();
}


/*
 *  fillVBox: fill the dialog's VBox
 */
void TFDialogErode::fillVBox ()
{
	SanityCheck::bailout ((!this->d_baseFrame), "TFDialogErode called without Frame label", 
		"TFDialogErode::setHFobjs");

	buildErodeFrame ();
	buildFlowmapFrame ();
}


/*
 *  buildDialogWindow:
 */
void TFDialogErode::buildDialogWindow ()
{
	this->set_usize (400, 450);
	this->set_title (d_winTitle);

	this->get_vbox()->set_border_width (2);

	setupVBox ();

	this->d_baseFrame->add (d_vBoxMain);
	fillVBox ();
	fillActionArea ();

	d_vBoxMain.show ();
	this->show ();
}


/* 
 * checkboxCallback: used so that d_CBsave can (de)activate d_entFileName
 */
void TFDialogErode::checkboxCallback ()
{	
	if (d_CBsave->get_active())
		d_entFileName.set_sensitive (TRUE);
	else
		d_entFileName.set_sensitive (FALSE);
}


void TFDialogErode::buttonCallbackOK ()
{	
	this->hide ();
	gdk_flush ();
	buttonCallbackApply ();
}


/*
 *  buttonCallbackApply: rotate the Height Field
 */
void TFDialogErode::buttonCallbackApply ()
{
	SanityCheck::bailout ((!d_HFO), "d_HFO==NULL", "TFDialogErode::buttonCallbackApply");

	int		nTurns, maxFlowAge;
	float		maxElvErode;
	char		*buf=NULL,
			*fName=NULL;
	bool		doSFD=TRUE, observeSea=FALSE, trimLocalPeaks=TRUE;

	nTurns = (int)(d_adjNTurns.get_value());
	maxFlowAge = (int)(d_adjMaxFlowAge.get_value());
	maxElvErode = d_adjMaxElvErode.get_value();

	doSFD = !d_RB2->get_active();
	observeSea = d_CBseaLevel->get_active();
	trimLocalPeaks = d_CBtrim->get_active();

	if (d_CBsave->get_active())
		{
		fName = const_cast<char*>(d_entFileName.get_text().data());
		fName[d_entFileName.get_text_length()]='\0';
		
		// convert filename to a sprintf mask
		if (fName && strlen(fName))
			{
			char 	*p;

			buf = new char[strlen(fName+10)];
			strcpy (buf, "");
			if ((p=strchr (fName, '.')))
				{
				if (!strstr (fName, "%d"))
					{
					buf = strncat (buf, fName, p-fName);
					buf = strcat (buf, "%d.");
					buf = strcat (buf, p+1);
					fName = buf;
					}
				}
			else
				{
				buf = strcat (buf, fName);
				buf = strcat (buf, "%d");
				buf = strcat (buf, ".tga");
				fName = buf;
				}
					
			}
		else
		if (fName)
			fName = NULL;
		}

	if ((nTurns > maxFlowAge) && (d_HF->getSealevel() < maxElvErode))
		{
		d_HFO->erode (nTurns, maxFlowAge, maxElvErode, fName, 
			trimLocalPeaks, doSFD, !observeSea, d_HFD);
		d_HFD->draw ();
		}
	else
		SanityCheck::warning ((TRUE), "maxFlowAge>nTurns", "TFDialogErode::buttonCallbackApply");

	if (buf)
		delete [] buf;
}


gint TFDialogErode::delete_event_impl (GdkEventAny *)
{
	this->hide ();
	return (0);
}


/*
int GlobalTrace::d_traceLevel=255;    // instantiate static data member
int main (int argc, char *argv[])
{
	Gtk_Main        gtkMain (&argc, &argv);
	TFDialogErode *testDialog;

	testDialog = new TFDialogErode (NULL, NULL, "TestTitle", "TestMessage");
	testDialog->show ();
	gtkMain.run ();
}
*/
