/* Terraform - (C) 1997-1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite s_HBOffs_HBOff0,
 * Boston, MA 02111-1s_HBOff07, USA.
 */


#include "TFDialogFill.h"
#include "GuiDialogOAC.h"
#include "GlobalSanityCheck.h"
#include "GlobalTrace.h"



/*
 *  constructor: initialize all data members 
 */
TFDialogFill::TFDialogFill (HeightField *HF, HeightFieldDraw *HFD)
	        : GuiDialogPreview ("Terraform Fill Dialog", 
				    "Fill Parameters", 6, 4),
		  d_vBoxOptions (TRUE, 0),
		  d_hBoxFillElv (TRUE, 0),
		  d_hBoxTightFactor (TRUE, 0),
		  d_hBoxKeepMin (TRUE, 0),
		  d_adjFillElv (0.33, 0.0, 1.0, 0.01),
		  d_adjTightFactor (0.5, 0.0, 1.0, 0.01),
		  d_hsFillElv (d_adjFillElv),
		  d_hsTightFactor (d_adjTightFactor),
		  d_lblFillElv ("Elevation to fill up to"),
		  d_lblTightFactor ("Tightness factor"),
		  d_lblKeepMin ("Keep zero elevation"),
		  d_optionsFrame ("Fill Options")

{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFDialogFill\n");

	d_HFO = NULL;
	setHFobjs (HF, HFD);

	buildDialogWindow ();
}


/*
 *  destructor: clean up 
 */
TFDialogFill::~TFDialogFill ()
{
	if (d_HFO)
		delete d_HFO;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFDialogFill\n");
}


/*
 *  setHBobjs: set the HB objects we operate on 
 */
void TFDialogFill::setHFobjs (HeightField *HF, HeightFieldDraw *HFD)
{
	SanityCheck::bailout ((!HF), "HF==NULL", "TFDialogFill::setHFobjs");
	SanityCheck::bailout ((!HFD), "HFD==NULL", "TFDialogFill::setHFobjs");

	if (d_HFO)
		delete d_HFO;

	d_HF = HF;
	d_HFD = HFD;
	d_HFO = new HeightFieldOps (d_HF);
}


/*
 *  fillVBox: fill the dialog's VBox
 */
void TFDialogFill::insertOptions ()
{
	SanityCheck::bailout ((!this->d_baseFrame), "TFDialogFill called without Frame label",
		"TFDialogFill::setHFobjs");

	d_optionsFrame.set_shadow_type (GTK_SHADOW_ETCHED_IN);
	d_optionsFrame.set_border_width (5);

	d_hBoxFillElv.pack_start (d_lblFillElv, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_lblFillElv.show ();
	d_hsFillElv.set_update_policy (GTK_UPDATE_DELAYED);
	d_hsFillElv.set_digits (2);
	d_hsFillElv.set_draw_value (TRUE);
	d_hBoxFillElv.pack_end (d_hsFillElv, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_hsFillElv.show ();
	d_vBoxOptions.pack_start (d_hBoxFillElv, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_hBoxFillElv.show ();

	d_hBoxTightFactor.pack_start (d_lblTightFactor, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_lblTightFactor.show ();
	d_hsTightFactor.set_update_policy (GTK_UPDATE_DELAYED);
	d_hsTightFactor.set_digits (2);
	d_hsTightFactor.set_draw_value (TRUE);
	d_hBoxTightFactor.pack_end (d_hsTightFactor, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_hsTightFactor.show ();
	d_vBoxOptions.pack_start (d_hBoxTightFactor, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_hBoxTightFactor.show ();

	d_hBoxKeepMin.pack_start (d_lblKeepMin, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_lblKeepMin.show ();
	d_cbKeepMin.set_active (TRUE);
	d_hBoxKeepMin.pack_end (d_cbKeepMin, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_cbKeepMin.show ();
	d_vBoxOptions.pack_start (d_hBoxKeepMin, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_hBoxKeepMin.show ();
	

	d_optionsFrame.add (d_vBoxOptions);
	this->d_table->attach (d_optionsFrame, 3, 6, 1, 3);
	d_vBoxOptions.show ();
	d_optionsFrame.show ();
	this->d_baseFrame->show ();
	this->get_vbox()->show ();
}


/*
 *  buildDialogWindow:
 */
void TFDialogFill::buildDialogWindow ()
{
	this->set_usize (400, 225);
	this->set_title (d_winTitle);

	this->get_vbox()->set_border_width (2);

	this->setupVBox ();
	this->insertPreview ();
	insertOptions ();
	this->fillActionArea ();

	this->show ();
}


void TFDialogFill::buttonCallbackOK ()
{	
	this->hide ();
	gdk_flush ();
	buttonCallbackApply ();
}


/*
 *  buttonCallbackApply: fill the height field 
 */
void TFDialogFill::buttonCallbackApply ()
{
	float		elv, 
			factor; 
	bool		keepMin=d_cbKeepMin.get_active();

	elv = d_adjFillElv.get_value ();
	factor = d_adjTightFactor.get_value ();

	SanityCheck::bailout ((!d_HFO), "d_HFO==NULL", "TFDialogFill::buttonCallbackApply");
	d_HFO->fill (elv, factor, keepMin);
	d_HFD->draw ();
}


gint TFDialogFill::delete_event_impl (GdkEventAny *)
{
	this->hide ();
	return (0);
}


/*
int GlobalTrace::d_traceLevel=255;    // instantiate static data member
int main (int argc, char *argv[])
{
	Gtk_Main        gtkMain (&argc, &argv);
	TFDialogFill *testDialog;

	testDialog = new TFDialogFill (NULL, NULL, "TestTitle", "TestMessage");
	testDialog->show ();
	gtkMain.run ();
}
*/
