/* Terraform - (C) 1997-1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "TFDialogFlowmap.h"
#include "GlobalSanityCheck.h"
#include "GlobalTrace.h"


/*
 *  constructor: initialize all data members 
 */
TFDialogFlowmap::TFDialogFlowmap (HeightField *HF, HeightFieldDraw *HFD)
	        : GuiDialogOAC ("Terraform Flowmap Dialog", 
				"Flowmap Type"),
		  d_vBoxList (TRUE, 5),
		  d_vBoxMain (TRUE, 5)
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFDialogFlowmap\n");

	d_HFO = NULL;
	setHFobjs (HF, HFD);
	d_RB1 = d_RB2 = NULL;
	d_CB = NULL;
	d_group = NULL;

	buildDialogWindow ();
}


/*
 *  destructor: clean up 
 */
TFDialogFlowmap::~TFDialogFlowmap ()
{
	if (d_HFO) delete d_HFO;
	if (d_RB1) delete d_RB1;
	if (d_RB2) delete d_RB2;
	if (d_CB) delete d_CB;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFDialogFlowmap\n");
}


/*
 *  setHBobjs: set the HB objects we operate on 
 */
void TFDialogFlowmap::setHFobjs (HeightField *HF, HeightFieldDraw *HFD)
{
	SanityCheck::bailout ((!HF), "HF==NULL", "TFDialogFlowmap::setHFobjs");
	SanityCheck::bailout ((!HFD), "HFD==NULL", "TFDialogFlowmap::setHFobjs");

	if (HF==d_HF && HFD==d_HFD)
		return;

	if (d_HFO) delete d_HFO;

	d_HF = HF;
	d_HFD = HFD;
	d_HFO = new HeightFieldOps (d_HF);
}


/*
 *  fillVBox: fill the dialog's VBox
 */
void TFDialogFlowmap::fillVBox ()
{
	SanityCheck::bailout ((!this->d_baseFrame), "TFDialogFlowmap called without Frame label", 
		"TFDialogFlowmap::setHFobjs");

	d_RB1 = new Gtk_RadioButton ((GSList *)NULL, "Single Flow Direction");
	d_RB1->set_active (TRUE);
	d_vBoxList.pack_start (*d_RB1, TRUE, TRUE, s_VBOff);
	d_RB1->show ();

	d_group = d_RB1->group ();
	d_RB2 = new Gtk_RadioButton (d_group, "Multiple Flow Direction");
	d_vBoxList.pack_start (*d_RB2, TRUE, TRUE, s_VBOff);
	d_RB2->show ();

	this->d_baseFrame->add (d_vBoxList);

	d_CB = new Gtk_CheckButton ("Observe Sealevel");
	d_CB->set_active (FALSE);
	this->get_vbox()->pack_start (*d_CB, TRUE, TRUE, s_VBOff);
	d_CB->show ();

	d_vBoxList.show ();
	this->d_baseFrame->show ();
	this->get_vbox()->show ();
}


/*
 *  buildDialogWindow:
 */
void TFDialogFlowmap::buildDialogWindow ()
{
	this->set_usize (200, 150);
	this->set_title (d_winTitle);

	this->get_vbox()->set_border_width (2);

	this->setupVBox ();
	fillVBox ();
	fillActionArea ();

	this->show ();
}


void TFDialogFlowmap::buttonCallbackOK ()
{	
	this->hide ();
	gdk_flush ();
	buttonCallbackApply ();
}


/*
 *  buttonCallbackApply: rotate the Height Field
 */
void TFDialogFlowmap::buttonCallbackApply ()
{
	SanityCheck::bailout ((!d_HFO), "d_HFO==NULL", "TFDialogFlowmap::buttonCallbackApply");
	HeightField	*HFF=NULL;

	if (d_RB1->get_active())
		HFF = d_HFO->flowmap (TRUE, !d_CB->get_active());
	else
	if (d_RB2->get_active())
		HFF = d_HFO->flowmap (FALSE, !d_CB->get_active());

	d_HF->setData (HFF->getData(), HFF->getWidth(), HFF->getHeight());
		
	d_HFD->draw ();
}


gint TFDialogFlowmap::delete_event_impl (GdkEventAny *)
{
	this->hide ();
	return (0);
}


/*
int GlobalTrace::d_traceLevel=255;    // instantiate static data member
int main (int argc, char *argv[])
{
	Gtk_Main        gtkMain (&argc, &argv);
	TFDialogFlowmap *testDialog;

	testDialog = new TFDialogFlowmap (NULL, NULL, "TestTitle", "TestMessage");
	testDialog->show ();
	gtkMain.run ();
}
*/
