/* Terraform - (C) 1997-1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "TFDialogFold.h"
#include "GlobalSanityCheck.h"
#include "GlobalTrace.h"



/*
 *  constructor: initialize all data members 
 */
TFDialogFold::TFDialogFold (HeightField *HF, HeightFieldDraw *HFD)
	        : GuiDialogOAC ("Terraform Fold Dialog", 
				"Edge Fold Offset"),
		  d_hBoxFoldOffset (TRUE, 0),
		  d_adjFoldOffset (20, 10, 100, 1),
		  d_hsFoldOffset (d_adjFoldOffset),
		  d_lblFoldOffset ("Edge Fold Offset")

{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFDialogFold\n");

	d_HFO = NULL;
	setHFobjs (HF, HFD);

	buildDialogWindow ();
}


/*
 *  destructor: clean up 
 */
TFDialogFold::~TFDialogFold ()
{
	if (d_HFO)
		delete d_HFO;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFDialogFold\n");
}


/*
 *  setHBobjs: set the HB objects we operate on 
 */
void TFDialogFold::setHFobjs (HeightField *HF, HeightFieldDraw *HFD)
{
	SanityCheck::bailout ((!HF), "HF==NULL", "TFDialogFold::setHFobjs");
	SanityCheck::bailout ((!HFD), "HFD==NULL", "TFDialogFold::setHFobjs");

	if (d_HFO)
		delete d_HFO;

	d_HF = HF;
	d_HFD = HFD;
	d_HFO = new HeightFieldOps (d_HF);
}


/*
 *  fillVBox: fill the dialog's VBox
 */
void TFDialogFold::fillVBox ()
{
	SanityCheck::bailout ((!this->d_baseFrame), "TFDialogFold called without Frame label",
		"TFDialogFold::setHFobjs");

	d_hBoxFoldOffset.pack_start (d_lblFoldOffset, TRUE, TRUE, s_HBOff);
	d_lblFoldOffset.show ();
	d_hsFoldOffset.set_update_policy (GTK_UPDATE_DELAYED);
	d_hsFoldOffset.set_digits (0);
	d_hsFoldOffset.set_draw_value (TRUE);
	d_hBoxFoldOffset.pack_end (d_hsFoldOffset, TRUE, TRUE, s_HBOff);
	d_hsFoldOffset.show ();

	this->d_baseFrame->add (d_hBoxFoldOffset);
	d_hBoxFoldOffset.show ();
	this->d_baseFrame->show ();
	this->get_vbox()->show ();
}


/*
 *  buildDialogWindow:
 */
void TFDialogFold::buildDialogWindow ()
{
	this->set_usize (250, 125);
	this->set_title (d_winTitle);

	this->get_vbox()->set_border_width (2);

	this->setupVBox ();
	fillVBox ();
	fillActionArea ();

	this->show ();
}


void TFDialogFold::buttonCallbackOK ()
{	
	this->hide ();
	gdk_flush ();
	buttonCallbackApply ();
}


/*
 *  buttonCallbackApply: fold the height field 
 */
void TFDialogFold::buttonCallbackApply ()
{
	float			value; 

	value = d_adjFoldOffset.get_value ();

	SanityCheck::bailout ((!d_HFO), "d_HFO==NULL", "TFDialogFold::buttonCallbackApply");
	d_HFO->fold ((int)value);
	d_HFD->draw ();
}


gint TFDialogFold::delete_event_impl (GdkEventAny *)
{
	this->hide ();
	return (0);
}


/*
int GlobalTrace::d_traceLevel=255;    // instantiate static data member
int main (int argc, char *argv[])
{
	Gtk_Main        gtkMain (&argc, &argv);
	TFDialogFold *testDialog;

	testDialog = new TFDialogFold (NULL, NULL, "TestTitle", "TestMessage");
	testDialog->show ();
	gtkMain.run ();
}
*/
