/* Terraform - (C) 1997-1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include <time.h>
//#include <values.h>
#include "TFDialogGenSSynth.h"
#include "HeightFieldGenSSynth.h"
#include "MathRandom.h"
#include "TFGui.h"
#include "GlobalSanityCheck.h"
#include "GlobalTrace.h"




/*
 *  constructor: initialize all data members 
 */
TFDialogGenSSynth::TFDialogGenSSynth (HeightField *HF, TFGui *HFGui)
	        : GuiDialogOAC ("Specral Synthesis Generate Dialog", 
				"Generation Parameters"),
		  d_vBoxList (TRUE, 5),
		  d_hBoxSize (TRUE, 5),
		  d_hBoxDim (TRUE, 5),
		  d_hBoxInvert (TRUE, 5),
		  d_hBoxGenSeed (TRUE, 5),
		  d_hBoxSeed (TRUE, 5),
		  d_adjDim (2.5, 2.0, 2.99, 1),
		  d_adjSize (400, 10, 1000, 1),
		  d_hsDim (d_adjDim),
		  d_hsSize (d_adjSize),
		  d_lblSize ("Size"),
		  d_lblDim ("Fractal dimension"),
		  d_lblInvert ("Invert sin/cos"),
		  d_lblGenSeed ("Generate new seed"),
		  d_lblSeed ("SpecifySeed")
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFDialogGenSSynth\n");

	d_HFGen = NULL;
	d_seedInitStr = new char [40];
	setHFobjs (HF, HFGui);

	buildDialogWindow ();
}



/*
 *  destructor: clean up, free allocated memory
 */
TFDialogGenSSynth::~TFDialogGenSSynth ()
{
	if (d_HFGen) delete d_HFGen;
	if (d_seedInitStr) delete d_seedInitStr;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFDialogGenSSynth\n");
}


/*
 *  setHFobs: set the HF objects we operate on
 */
void TFDialogGenSSynth::setHFobjs (HeightField *HF, TFGui *HFGui)
{
	SanityCheck::bailout ((!HF), "HF==NULL", "TFDialogGenSSynth::setHFobjs");
	SanityCheck::bailout ((!HFGui), "HFGui==NULL", "TFDialogGenSSynth::setHFobjs");

	if (HF == d_HF && HFGui == d_HFGui)
		return;

	if (d_HFGen)
		delete d_HFGen;

	d_HF = HF;
	d_HFGui = HFGui;
	d_HFGen = new HeightFieldGenSSynth (d_HF);
}


/*
 *  fillVBox: fill the dialog's VBox
 */
void TFDialogGenSSynth::fillVBox ()
{
	SanityCheck::bailout ((!this->d_baseFrame), "TFDialogGenSSynth called without Frame label",
		"TFDialogGenSSynth::setHFobjs");

        d_hBoxSize.pack_start (d_lblSize, TRUE, TRUE, s_HBOff);
        d_lblSize.show ();
        d_hsSize.set_update_policy (GTK_UPDATE_DELAYED);
	d_hsSize.set_adjustment (d_adjSize);
        d_hsSize.set_digits (0);
        d_hsSize.set_draw_value (TRUE);
        d_hBoxSize.pack_end (d_hsSize, TRUE, TRUE, s_HBOff);
        d_hsSize.show ();
	d_vBoxList.pack_start (d_hBoxSize, TRUE, TRUE, s_VBOff);
	d_hBoxSize.show ();

        d_hBoxDim.pack_start (d_lblDim, TRUE, TRUE, s_HBOff);
        d_lblDim.show ();
        d_hsDim.set_update_policy (GTK_UPDATE_DELAYED);
	d_hsDim.set_adjustment (d_adjDim);
        d_hsDim.set_digits (2);
        d_hsDim.set_draw_value (TRUE);
        d_hBoxDim.pack_end (d_hsDim, TRUE, TRUE, s_HBOff);
        d_hsDim.show ();
	d_vBoxList.pack_start (d_hBoxDim, TRUE, TRUE, s_VBOff);
	d_hBoxDim.show ();

	d_hBoxInvert.pack_start (d_lblInvert, TRUE, TRUE, s_HBOff);
	d_lblInvert.show ();
	d_hBoxInvert.pack_end (d_btnInvert, TRUE, TRUE, s_HBOff);
	d_btnInvert.show ();
	d_vBoxList.pack_start (d_hBoxInvert, TRUE, TRUE, s_VBOff);
	d_hBoxInvert.show ();

	d_vBoxList.pack_start (d_sepMain, TRUE, TRUE, s_HBOff);
	d_sepMain.show ();

	d_hBoxGenSeed.pack_start (d_lblGenSeed, TRUE, TRUE, s_HBOff);
	d_lblGenSeed.show ();
	d_hBoxGenSeed.pack_end (d_btnGenSeed, TRUE, TRUE, s_HBOff);
	d_btnGenSeed.set_active (TRUE);
	d_btnGenSeed.show ();
	connect_to_method (d_btnGenSeed.clicked, this, &checkboxCallback);
	d_vBoxList.pack_start (d_hBoxGenSeed, TRUE, TRUE, s_VBOff);
	d_hBoxGenSeed.show ();

	d_lblSeed.set_justify (GTK_JUSTIFY_RIGHT);
	d_hBoxSeed.pack_start (d_lblSeed, TRUE, TRUE, s_HBOff);
	d_lblSeed.set_sensitive (FALSE);
	d_lblSeed.show ();

	MathRandom	*mRand = new MathRandom ();
	sprintf (d_seedInitStr, "%d", (int)mRand->rnd());
	delete mRand;
	d_enSeed.set_text (d_seedInitStr);
	d_enSeed.set_sensitive (FALSE);
	d_hBoxSeed.pack_end (d_enSeed, TRUE, TRUE, s_HBOff);
	d_enSeed.show ();
	d_vBoxList.pack_start (d_hBoxSeed, TRUE, TRUE, s_VBOff);
	d_hBoxSeed.show ();

	this->d_baseFrame->add (d_vBoxList);
	d_vBoxList.show ();
	this->d_baseFrame->show ();
	this->get_vbox()->show ();
	
}


/*
 *  buildDialogWindow:
 */
void TFDialogGenSSynth::buildDialogWindow ()
{
	this->set_usize (300, 280);
	this->set_title (this->d_winTitle);

	this->get_vbox()->set_border_width (2);

	this->setupVBox ();
	fillVBox ();
	fillActionArea ();

	this->show ();
}


void TFDialogGenSSynth::checkboxCallback ()
{
	bool		status=d_enSeed.is_sensitive ();

	d_lblSeed.set_sensitive (!status);
	d_enSeed.set_sensitive (!status);
}


void TFDialogGenSSynth::buttonCallbackOK ()
{	
	this->hide ();
	gdk_flush ();
	buttonCallbackApply ();
}


void TFDialogGenSSynth::buttonCallbackApply ()
{
	float			H, dim; 	// Dimension = 3-h
	char			*s;
	int			size, seed;
	bool			invert, magnitude;

	dim = d_adjDim.get_value (); 	// 2 .. 3
	H = 3-dim;

	size = (int)(d_adjSize.get_value());

	if (d_btnGenSeed.get_active())
		{
		MathRandom	*mRand = new MathRandom ();

		seed = (int) (mRand->rnd()*G_MAXINT);
		sprintf (d_seedInitStr, "%d", seed);
		d_enSeed.set_text (d_seedInitStr);
		delete mRand;
		}
	else
		{
		s = const_cast<char*>(d_enSeed.get_text().data());
		seed = atoi (s);
		}

	invert = d_btnInvert.get_active ();

	magnitude = d_btnGenSeed.get_active ();

	SanityCheck::bailout ((!d_HFGen), "d_HFGen==NULL", "TFDialogGenSSynth::buttonCallbackApply");
	d_HFGen->generate (size, H, seed, invert);
	d_HFGui->setHeightField (d_HF);
}


gint TFDialogGenSSynth::delete_event_impl (GdkEventAny *)
{
	this->hide ();
	return (0);
}



/*
int GlobalTrace::d_traceLevel=255;    // instantiate static data member
int main (int argc, char *argv[])
{
	Gtk_Main        gtkMain (&argc, &argv);
	TFDialogGenSSynth *testDialog;

	testDialog = new TFDialogGenSSynth (NULL, NULL, "TestTitle", "TestMessage");
	testDialog->show ();
	gtkMain.run ();
}
*/
