/* Terraform - (C) 1997-1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include <time.h>
//#include <values.h>
#include "TFDialogGenSubdiv.h"
#include "HeightFieldGenSubdiv.h"
#include "MathRandom.h"
#include "TFGui.h"
#include "GlobalSanityCheck.h"
#include "GlobalTrace.h"
#include "MathRandom.h"




/*
 *  constructor: initialize all data members 
 */
TFDialogGenSubdiv::TFDialogGenSubdiv (HeightField *HF, TFGui *HFGui)
	        : GuiDialogOAC ("Generation by Subdivision Dialog", 
				"Parameters"),
		  d_vBoxList (TRUE, 5),
		  d_hBoxMethod (TRUE, 5),
		  d_hBoxSize (TRUE, 5),
		  d_hBoxScaleFactor (TRUE, 5),
		  d_hBoxGenSeed (TRUE, 5),
		  d_hBoxSeed (TRUE, 5),
		  d_adjScaleFactor (0.5, 0.01, 1, 0.01),
		  d_adjSize (400, 10, 1000, 1),
		  d_hsScaleFactor (d_adjScaleFactor),
		  d_hsSize (d_adjSize),
		  d_lblMethod ("Method to use"),
		  d_lblSize ("Size"),
		  d_lblScaleFactor ("Scale Factor"),
		  d_lblGenSeed ("Generate new seed"),
		  d_lblSeed ("SpecifySeed")
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFDialogGenSubdiv\n");

	d_HFGen = NULL;
	d_seedInitStr = new char [40];
	d_glstMethod = NULL;
	d_mode = 0;
	setHFobjs (HF, HFGui);

	buildDialogWindow ();
}



/*
 *  destructor: clean up, free allocated memory
 */
TFDialogGenSubdiv::~TFDialogGenSubdiv ()
{
	if (d_HFGen) delete d_HFGen;
	if (d_seedInitStr) delete d_seedInitStr;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFDialogGenSubdiv\n");
}


/*
 *  setHFobs: set the HF objects we operate on
 */
void TFDialogGenSubdiv::setHFobjs (HeightField *HF, TFGui *HFGui)
{
	SanityCheck::bailout ((!HF), "HF==NULL", "TFDialogGenSubdiv::setHFobjs");
	SanityCheck::bailout ((!HFGui), "HFGui==NULL", "TFDialogGenSubdiv::setHFobjs");

	if (HF == d_HF && HFGui == d_HFGui)
		return;

	if (d_HFGen)
		delete d_HFGen;

	d_HF = HF;
	d_HFGui = HFGui;
	d_HFGen = new HeightFieldGenSubdiv (d_HF);
}


/*
 *  fillVBox: fill the dialog's VBox
 */
void TFDialogGenSubdiv::fillVBox ()
{
	SanityCheck::bailout ((!this->d_baseFrame), "TFDialogGenSubdiv called without Frame label",
		"TFDialogGenSubdiv::setHFobjs");

        d_hBoxMethod.pack_start (d_lblMethod, TRUE, TRUE, s_HBOff);
        d_lblMethod.show ();

	// build Option Menu

	d_glstMethod = g_list_append (d_glstMethod, MENU_GEN_RECURSIVE);
	d_glstMethod = g_list_append (d_glstMethod, MENU_GEN_OFFSET);
	d_glstMethod = g_list_append (d_glstMethod, MENU_GEN_MIDPOINT);
	d_glstMethod = g_list_append (d_glstMethod, MENU_GEN_DIAMOND);

	d_cmbMethod.set_popdown_strings (d_glstMethod);
	d_cmbMethod.get_entry()->set_text (MENU_GEN_RECURSIVE);
	d_cmbMethod.get_entry()->set_editable (FALSE);
	
	d_cmbMethod.show ();
        d_hBoxMethod.pack_start (d_cmbMethod, TRUE, TRUE, s_HBOff);
	d_vBoxList.pack_start (d_hBoxMethod, TRUE, TRUE, s_VBOff);
	d_hBoxMethod.show ();

	// start actual generation parameters
        d_hBoxSize.pack_start (d_lblSize, TRUE, TRUE, s_HBOff);
        d_lblSize.show ();
        d_hsSize.set_update_policy (GTK_UPDATE_DELAYED);
	d_hsSize.set_adjustment (d_adjSize);
        d_hsSize.set_digits (0);
        d_hsSize.set_draw_value (TRUE);
        d_hBoxSize.pack_end (d_hsSize, TRUE, TRUE, s_HBOff);
        d_hsSize.show ();
	d_vBoxList.pack_start (d_hBoxSize, TRUE, TRUE, s_VBOff);
	d_hBoxSize.show ();

        d_hBoxScaleFactor.pack_start (d_lblScaleFactor, TRUE, TRUE, s_HBOff);
        d_lblScaleFactor.show ();
        d_hsScaleFactor.set_update_policy (GTK_UPDATE_DELAYED);
	d_hsScaleFactor.set_adjustment (d_adjScaleFactor);
        d_hsScaleFactor.set_digits (2);
        d_hsScaleFactor.set_draw_value (TRUE);
        d_hBoxScaleFactor.pack_end (d_hsScaleFactor, TRUE, TRUE, s_HBOff);
        d_hsScaleFactor.show ();
	d_vBoxList.pack_start (d_hBoxScaleFactor, TRUE, TRUE, s_VBOff);
	d_hBoxScaleFactor.show ();

	d_vBoxList.pack_start (d_sepMain, TRUE, TRUE, s_HBOff);
	d_sepMain.show ();

	// random seed parameters 
	d_hBoxGenSeed.pack_start (d_lblGenSeed, TRUE, TRUE, s_HBOff);
	d_lblGenSeed.show ();
	d_hBoxGenSeed.pack_end (d_btnGenSeed, TRUE, TRUE, s_HBOff);
	d_btnGenSeed.set_active (TRUE);
	d_btnGenSeed.show ();
	connect_to_method (d_btnGenSeed.clicked, this, &seedCheckboxCallback);
	d_vBoxList.pack_start (d_hBoxGenSeed, TRUE, TRUE, s_VBOff);
	d_hBoxGenSeed.show ();

	d_lblSeed.set_justify (GTK_JUSTIFY_RIGHT);
	d_hBoxSeed.pack_start (d_lblSeed, TRUE, TRUE, s_HBOff);
	d_lblSeed.set_sensitive (FALSE);
	d_lblSeed.show ();

	MathRandom	*mRand = new MathRandom ();
	sprintf (d_seedInitStr, "%d", (int)mRand->rnd());
	delete mRand;
	d_enSeed.set_text (d_seedInitStr);
	d_enSeed.set_sensitive (FALSE);
	d_hBoxSeed.pack_end (d_enSeed, TRUE, TRUE, s_HBOff);
	d_enSeed.show ();
	d_vBoxList.pack_start (d_hBoxSeed, TRUE, TRUE, s_VBOff);
	d_hBoxSeed.show ();

	this->d_baseFrame->add (d_vBoxList);
	d_vBoxList.show ();
	this->d_baseFrame->show ();
	this->get_vbox()->show ();
	
}


/*
 *  buildDialogWindow:
 */
void TFDialogGenSubdiv::buildDialogWindow ()
{
	this->set_usize (300, 300);
	this->set_title (this->d_winTitle);

	this->get_vbox()->set_border_width (2);

	this->setupVBox ();
	fillVBox ();
	fillActionArea ();

	this->show ();
}


void TFDialogGenSubdiv::seedCheckboxCallback ()
{
	bool		status=d_enSeed.is_sensitive ();

	d_lblSeed.set_sensitive (!status);
	d_enSeed.set_sensitive (!status);
}


void TFDialogGenSubdiv::buttonCallbackOK ()
{	
	this->hide ();
	gdk_flush ();
	buttonCallbackApply ();
}


void TFDialogGenSubdiv::buttonCallbackApply ()
{
	char			*s;
	int			size, seed;
	float			scaleFactor;
	string			entry;

	size = (int)(d_adjSize.get_value());
	scaleFactor = d_adjScaleFactor.get_value();

	if (d_btnGenSeed.get_active())
		{
		MathRandom	*mRand = new MathRandom ();

		seed = (int) (mRand->rnd()*G_MAXINT);
		sprintf (d_seedInitStr, "%d", seed);
		d_enSeed.set_text (d_seedInitStr);
		delete mRand;
		}
	else
		{
		s = const_cast<char*>(d_enSeed.get_text().data());
		seed = atoi (s);
		}

	SanityCheck::bailout ((!d_HFGen), "d_HFGen==NULL", "TFDialogGenSubdiv::buttonCallbackApply");

	entry = (d_cmbMethod.get_entry())->get_text();
	if (entry == MENU_GEN_DIAMOND)
		d_HFGen->generateDiamond (size, scaleFactor, seed);
	else
	if (entry == MENU_GEN_MIDPOINT)
		d_HFGen->generateMidpoint (size, scaleFactor, seed);
	else
	if (entry == MENU_GEN_OFFSET)
		d_HFGen->generateOffset (size, scaleFactor, seed);
	else
	if (entry == MENU_GEN_RECURSIVE)
		d_HFGen->generateRecursive (size, scaleFactor, seed);
	else
		SanityCheck::bailout ((TRUE), "uncaught RadioMenuItem", "TFDialogGenSubdiv::buttonApplyCallback");

	
	d_HFGui->setHeightField (d_HF);
}


gint TFDialogGenSubdiv::delete_event_impl (GdkEventAny *)
{
	this->hide ();
	return (0);
}



/*
int GlobalTrace::d_traceLevel=255;    // instantiate static data member
int main (int argc, char *argv[])
{
	Gtk_Main        gtkMain (&argc, &argv);
	TFDialogGenSubdiv *testDialog;

	testDialog = new TFDialogGenSubdiv (NULL, NULL, "TestTitle", "TestMessage");
	testDialog->show ();
	gtkMain.run ();
}
*/
