/* Terraform - (C) 1997-1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "TFDialogLinearScale.h"
#include "GlobalSanityCheck.h"
#include "GlobalTrace.h"


/*
 *  constructor: initialize all data members 
 */
TFDialogLinearScale::TFDialogLinearScale (HeightField *HF, HeightFieldDraw *HFD)
	        : GuiDialogOAC ("Terraform Linear Scale Dialog", "Parameters"),
		  d_vbList (TRUE, 5),
		  d_hbCX (TRUE, 5),
		  d_hbCY (TRUE, 5),
		  d_hbScaleFactor (TRUE, 5),
		  d_hbMinDist (TRUE, 5),
		  d_hbMaxDist (TRUE, 5),
		  d_hbSmoothDist (TRUE, 5),
		  d_hbInvert (TRUE, 5),
		  d_hbFullDist (TRUE, 5),
		  d_lblCX ("Center X"),
		  d_lblCY ("Center Y"),
		  d_lblScaleFactor ("Scale Factor"),
		  d_lblMinDist ("Scale Etart Distance"),
		  d_lblMaxDist ("Scale End Distance"),
		  d_lblSmoothDist ("Smoothing Factor"),
		  d_lblInvert ("Invert Scale Factor"),
		  d_lblFullDist ("Scale against Max Distance"),
		  d_adjCX (0.5, 0, 1, 0.01),
		  d_adjCY (0.5, 0, 1, 0.01),
		  d_adjScaleFactor (1.5, 1, 2, 0.01),
		  d_adjMinDist (0.33, 0, 1, 0.01),
		  d_adjMaxDist (0.66, 0, 1, 0.01),
		  d_adjSmoothDist (0.1, 0, 1, 0.01),
		  d_hsCX (d_adjCX),
		  d_hsCY (d_adjCY),
		  d_hsScaleFactor (d_adjScaleFactor),
		  d_hsMinDist (d_adjMinDist),
		  d_hsMaxDist (d_adjMaxDist),
		  d_hsSmoothDist (d_adjSmoothDist),
		  d_btnInvert (),
		  d_btnFullDist ()
		  //d_btnInvert ("Invert scale factor"),
		  //d_btnFullDist ("Scale gainst max distance")
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFDialogLinearScale\n");

	d_HFO = NULL;
	setHFobjs (HF, HFD);

	buildDialogWindow ();
}


/*
 *  destructor: clean up 
 */
TFDialogLinearScale::~TFDialogLinearScale ()
{
	if (d_HFO)
		delete d_HFO;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFDialogLinearScale\n");
}


/*
 *  setHFobjs: set the HF objects we operate on
 */
void TFDialogLinearScale::setHFobjs (HeightField *HF, HeightFieldDraw *HFD)
{
	SanityCheck::bailout ((!HF), "HF==NULL", "TFDialogLinearScale::setHFobjs");
	SanityCheck::bailout ((!HFD), "HFD==NULL", "TFDialogLinearScale::setHFobjs");

	if (d_HFO)
		delete d_HFO;

	d_HF = HF;
	d_HFD = HFD;
	d_HFO = new HeightFieldOps (d_HF);
}


/*
 *  fillVBox: fill the dialog's VBox
 */
void TFDialogLinearScale::fillVBox()
{
	d_hbCX.pack_start (d_lblCX, TRUE, TRUE, s_HBOff);
	d_lblCX.show ();
	d_hsCX.set_update_policy (GTK_UPDATE_DELAYED);
	d_hsCX.set_digits (2);
	d_hsCX.set_draw_value (TRUE);
	d_hbCX.pack_end (d_hsCX, TRUE, TRUE, s_HBOff);
	d_hsCX.show ();
	d_vbList.pack_start (d_hbCX, TRUE, TRUE, s_VBOff);
	d_hbCX.show ();

	d_hbCY.pack_start (d_lblCY, TRUE, TRUE, s_HBOff);
	d_lblCY.show ();
	d_hsCY.set_update_policy (GTK_UPDATE_DELAYED);
	d_hsCY.set_digits (2);
	d_hsCY.set_draw_value (TRUE);
	d_hbCY.pack_end (d_hsCY, TRUE, TRUE, s_HBOff);
	d_hsCY.show ();
	d_vbList.pack_start (d_hbCY, TRUE, TRUE, s_VBOff);
	d_hbCY.show ();

	d_hbScaleFactor.pack_start (d_lblScaleFactor, TRUE, TRUE, s_HBOff);
	d_lblScaleFactor.show ();
	d_hsScaleFactor.set_update_policy (GTK_UPDATE_DELAYED);
	d_hsScaleFactor.set_digits (2);
	d_hsScaleFactor.set_draw_value (TRUE);
	d_hbScaleFactor.pack_end (d_hsScaleFactor, TRUE, TRUE, s_HBOff);
	d_hsScaleFactor.show ();
	d_vbList.pack_start (d_hbScaleFactor, TRUE, TRUE, s_VBOff);
	d_hbScaleFactor.show ();

	d_hbMinDist.pack_start (d_lblMinDist, TRUE, TRUE, s_HBOff);
	d_lblMinDist.show ();
	d_hsMinDist.set_update_policy (GTK_UPDATE_DELAYED);
	d_hsMinDist.set_digits (2);
	d_hsMinDist.set_draw_value (TRUE);
	d_hbMinDist.pack_end (d_hsMinDist, TRUE, TRUE, s_HBOff);
	d_hsMinDist.show ();
	d_vbList.pack_start (d_hbMinDist, TRUE, TRUE, s_VBOff);
	d_hbMinDist.show ();

	d_hbMaxDist.pack_start (d_lblMaxDist, TRUE, TRUE, s_HBOff);
	d_lblMaxDist.show ();
	d_hsMaxDist.set_update_policy (GTK_UPDATE_DELAYED);
	d_hsMaxDist.set_digits (2);
	d_hsMaxDist.set_draw_value (TRUE);
	d_hbMaxDist.pack_end (d_hsMaxDist, TRUE, TRUE, s_HBOff);
	d_hsMaxDist.show ();
	d_vbList.pack_start (d_hbMaxDist, TRUE, TRUE, s_VBOff);
	d_hbMaxDist.show ();

	d_hbSmoothDist.pack_start (d_lblSmoothDist, TRUE, TRUE, s_HBOff);
	d_lblSmoothDist.show ();
	d_hsSmoothDist.set_update_policy (GTK_UPDATE_DELAYED);
	d_hsSmoothDist.set_digits (2);
	d_hsSmoothDist.set_draw_value (TRUE);
	d_hbSmoothDist.pack_end (d_hsSmoothDist, TRUE, TRUE, s_HBOff);
	d_hsSmoothDist.show ();
	d_vbList.pack_start (d_hbSmoothDist, TRUE, TRUE, s_VBOff);
	d_hbSmoothDist.show ();

	d_hbInvert.pack_start (d_lblInvert, TRUE, TRUE, s_HBOff);
	d_lblInvert.show ();
	d_hbInvert.pack_end (d_btnInvert, TRUE, TRUE, s_HBOff);
	d_btnInvert.show ();
	d_vbList.pack_start (d_hbInvert, TRUE, TRUE, s_VBOff);
	d_hbInvert.show ();

	d_hbFullDist.pack_start (d_lblFullDist, TRUE, TRUE, s_HBOff);
	d_lblFullDist.show ();
	d_hbFullDist.pack_end (d_btnFullDist, TRUE, TRUE, s_HBOff);
	d_btnFullDist.show ();
	d_vbList.pack_start (d_hbFullDist, TRUE, TRUE, s_VBOff);
	d_hbFullDist.show ();
	
	this->d_baseFrame->add (d_vbList);
        d_vbList.show ();
        this->d_baseFrame->show ();
        this->get_vbox()->show ();
}



void TFDialogLinearScale::buildDialogWindow ()
{
	this->set_usize (300, 370);
	this->set_title (this->d_winTitle);

	this->get_vbox()->set_border_width (2);

	this->setupVBox ();
	this->fillVBox ();
	fillActionArea ();
	
	this->show ();
}


void TFDialogLinearScale::buttonCallbackOK ()
{	
	this->hide ();
	gdk_flush ();
	buttonCallbackApply ();
}


/*
 *  buttonCallbackApply: roughen the Height Field
 */
void TFDialogLinearScale::buttonCallbackApply ()
{
	float			cx, cy, sf, mind, maxd, smoothf;
	bool			invert, fulldist;

	cx = d_adjCX.get_value ();
	cy = d_adjCY.get_value ();
        sf = d_adjScaleFactor.get_value ();
        mind = d_adjMinDist.get_value (); 
	maxd = d_adjMaxDist.get_value ();
	smoothf = d_adjSmoothDist.get_value ();
	invert = d_btnInvert.get_active ();
	fulldist = d_btnFullDist.get_active ();

	if (invert)
		sf = 1-(sf-1);

	SanityCheck::bailout ((!d_HFO), "d_HFO==NULL", "TFDialogLinearScale::buttonCallbackApply");
	d_HFO->linearScale (cx, cy, sf, mind, maxd, smoothf, invert, fulldist);
	d_HFD->draw ();
}


gint TFDialogLinearScale::delete_event_impl (GdkEventAny *)
{
	this->hide ();
	return (0);
}


/*
int GlobalTrace::d_traceLevel=255;    // instantiate static data member
int main (int argc, char *argv[])
{
	Gtk_Main        gtkMain (&argc, &argv);
	TFDialogLinearScale *testDialog;

	testDialog = new TFDialogLinearScale (NULL, NULL, "TestTitle", "TestMessage");
	testDialog->show ();
	gtkMain.run ();
}
*/
