/* Terraform - (C) 1997-1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "TFDialogMirror.h"
#include "GlobalSanityCheck.h"
#include "GlobalTrace.h"


/*
 *  constructor: initialize all data members 
 */
TFDialogMirror::TFDialogMirror (HeightField *HF, HeightFieldDraw *HFD)
	        : GuiDialogOAC ("Terraform Mirror Dialog", 
				"Mirror along which Axis"),
		  d_vBoxList (TRUE, 5)
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFDialogMirror\n");

	d_HFO = NULL;
	setHFobjs (HF, HFD);
	d_RB1 = d_RB2 = d_RB3 = d_RB4 = NULL;
	d_group = NULL;

	buildDialogWindow ();
}


/*
 *  destructor: clean up 
 */
TFDialogMirror::~TFDialogMirror ()
{
	if (d_HFO) delete d_HFO;
	if (d_RB1) delete d_RB1;
	if (d_RB2) delete d_RB2;
	if (d_RB3) delete d_RB3;
	if (d_RB4) delete d_RB4;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFDialogMirror\n");
}


/*
 *  setHBobjs: set the HB objects we operate on 
 */
void TFDialogMirror::setHFobjs (HeightField *HF, HeightFieldDraw *HFD)
{
	SanityCheck::bailout ((!HF), "HF==NULL", "TFDialogMirror::setHFobjs");
	SanityCheck::bailout ((!HFD), "HFD==NULL", "TFDialogMirror::setHFobjs");

	if (d_HFO)
		delete d_HFO;

	d_HF = HF;
	d_HFD = HFD;
	d_HFO = new HeightFieldOps (d_HF);
}


/*
 *  fillVBox: fill the dialog's VBox
 */
void TFDialogMirror::fillVBox ()
{
	SanityCheck::bailout ((!this->d_baseFrame), "TFDialogMirror called without Frame label",
		"TFDialogMirror::setHFobjs");

	d_RB1 = new Gtk_RadioButton ((GSList *)NULL, "Horizonal Axis (-)");
	d_RB1->set_active(TRUE);
	d_vBoxList.pack_start (*d_RB1, TRUE, TRUE, s_VBOff);
	d_RB1->show ();

	d_group = d_RB1->group ();
	d_RB2 = new Gtk_RadioButton (d_group, "Vertical Axis (|)");
	d_vBoxList.pack_start (*d_RB2, TRUE, TRUE, s_VBOff);
	d_RB2->show ();

	d_group = d_RB2->group ();
	d_RB3 = new Gtk_RadioButton (d_group, "Top Left - Bottom Right (\\)");
	d_vBoxList.pack_start (*d_RB3, TRUE, TRUE, s_VBOff);
	d_RB3->show ();

	d_group = d_RB3->group ();
	d_RB4 = new Gtk_RadioButton (d_group, "Bottom Left - Top Right (/)");
	d_vBoxList.pack_start (*d_RB4, TRUE, TRUE, s_VBOff);
	d_RB4->show ();

	this->d_baseFrame->add (d_vBoxList);
	d_vBoxList.show ();
	this->d_baseFrame->show ();
	this->get_vbox()->show ();
}


/*
 *  buildDialogWindow:
 */
void TFDialogMirror::buildDialogWindow ()
{
	this->set_usize (200, 175);
	this->set_title (this->d_winTitle);

	this->get_vbox()->set_border_width (2);

	this->setupVBox ();
	fillVBox ();
	fillActionArea ();

	this->show ();
}


void TFDialogMirror::buttonCallbackOK ()
{	
	this->hide ();
	gdk_flush ();
	buttonCallbackApply ();
}


/*
 *  buttonCallbackApply: rotate the Height Field
 */
void TFDialogMirror::buttonCallbackApply ()
{
	SanityCheck::bailout ((!d_HFO), "d_HFO==NULL", "TFDialogMirror::buttonCallbackApply");

	if (d_RB1->get_active())
		d_HFO->mirror (1);
	else
	if (d_RB2->get_active())
		d_HFO->mirror (2);
	else
	if (d_RB3->get_active())
		d_HFO->mirror (3);
	else
	if (d_RB4->get_active())
		d_HFO->mirror (4);
		
	d_HFD->draw ();
}


gint TFDialogMirror::delete_event_impl (GdkEventAny *)
{
	this->hide ();
	return (0);
}


/*
int GlobalTrace::d_traceLevel=255;    // instantiate static data member
int main (int argc, char *argv[])
{
	Gtk_Main        gtkMain (&argc, &argv);
	TFDialogMirror *testDialog;

	testDialog = new TFDialogMirror (NULL, NULL, "TestTitle", "TestMessage");
	testDialog->show ();
	gtkMain.run ();
}
*/
