/* Terraform - (C) 1997-1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "TFDialogOptions.h"
#include "GlobalTrace.h"
#include "GlobalSanityCheck.h"


/*
 *  constructor: initialize all data members 
 */
TFDialogOptions::TFDialogOptions (HeightField *HF, HeightFieldDraw *HFD)
	 : GuiDialogOAC ("Terraform Options Dialog", NULL),
	   d_frame1 ("Global Draw Options"), 
	   d_frame2("Height Field Draw Options"),
	   d_frame3("Fast Wireframe Options"),
	   d_tabTitle1 ("Global"),
	   d_tabTitle2 ("Height Field View"),
	   d_tabTitle3 ("Fast Wireframe"),
	   d_msgScale ("Display Scale"),
	   d_msgSync ("Sync Rate (in lines)"),
	   d_msgResolution ("Wireframe Resolution"),
	   d_msgYScale ("Y Scale Factor"),
	   d_msgSealevel ("Sealevel"),
	   d_msgFillSea ("Sea is Filled"),
	   d_msgFastResolution ("Half Wireframe Resoltion"),
	   d_msgFastYscale ("Half Y Scale Factor"),
	   d_msgFastZrot ("Enable Z-axis rotation"),
	   d_vbox1 (TRUE, 2),
	   d_vbox2 (TRUE, 2),
	   d_vbox3 (TRUE, 2),
	   d_hbox1 (TRUE, 5),
	   d_hbox2 (TRUE, 5),
	   d_hbox3 (TRUE, 5),
	   d_hbox4 (TRUE, 5),
	   d_hbox5 (TRUE, 5),
	   d_hbox6 (TRUE, 5),
	   d_hbox7 (TRUE, 5),
	   d_hbox8 (TRUE, 5),
	   d_hbox9 (TRUE, 5),
	   d_adjScale (1.0, 1.0, 3.0, 1.0),
	   d_adjSync (1, 1, 5000, 1),
	   d_adjResolution (5.0, 1.0, 10.0, 1.0),
	   d_adjYScale (0.33, 0, 1, 0.01, 10, 0),
	   d_adjSealevel (0.33, 0, 1, 0.01, 10, 0),
	   d_hsScale (d_adjScale),
	   d_hsSync (d_adjSync),
	   d_hsResolution (d_adjResolution),
	   d_hsYScale (d_adjYScale),
	   d_hsSealevel (d_adjSealevel)
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFDialogOptions\n");

	setHFobjs (HF, HFD);

	buildDialogWindow ();
	getValues (&d_fillSea, &d_scale, &d_syncRate, &d_resolution, &d_yscale, &d_sealevel, 
		   &d_fastHalfRes, &d_fastHalfYscale, &d_fastZrot);
}


/*
 *  destructor: clean up
 */
TFDialogOptions::~TFDialogOptions ()
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFDialogOptions\n");
}


/*
 *  setHFobjs: set the HF obejcts we operate on 
 */
void TFDialogOptions::setHFobjs (HeightField *HF, HeightFieldDraw *HFD)
{
	SanityCheck::bailout ((!HF), "HF==NULL", "TFDialogOptions::setHFobjs");
	SanityCheck::bailout ((!HFD), "HFD==NULL", "TFDialogOptions::setHFobjs");

	d_HF = HF;
	d_HFD = HFD;
}


/*
 *  fillVBox: fill the dialog's VBox
 */
void TFDialogOptions::fillVBox ()
{
	this->get_vbox()->pack_start (d_notebook, TRUE, TRUE, 0);
	d_frame1.set_shadow_type (GTK_SHADOW_ETCHED_IN);
	d_frame2.set_shadow_type (GTK_SHADOW_ETCHED_IN);
	d_frame3.set_shadow_type (GTK_SHADOW_ETCHED_IN);
	d_notebook.append_page (d_frame1, d_tabTitle1);
	d_notebook.append_page (d_frame2, d_tabTitle2);
	d_notebook.append_page (d_frame3, d_tabTitle3);
	d_notebook.show ();

	buildDrawOptionsFrame ();
	buildHFDrawOptionsFrame ();
	buildFastWireOptionsFrame ();
}


/*
 *  buildDialogWindow:
 */
void TFDialogOptions::buildDialogWindow ()
{
	this->set_usize (300, 200);
	this->set_title (d_winTitle);

	this->get_vbox()->set_border_width (2);

	this->setupVBox ();
	fillVBox ();
	fillActionArea ();

	this->show ();
}


/*
 *  buildDrawOptionsFrame: frame for the global draw options
 */
void TFDialogOptions::buildDrawOptionsFrame ()
{
	d_msgScale.set_justify (GTK_JUSTIFY_LEFT);
	d_hbox1.pack_start (d_msgScale, TRUE, TRUE, s_HBOff);
	d_msgScale.show ();
        d_hsScale.set_update_policy (GTK_UPDATE_DELAYED);
        d_hsScale.set_digits (1);
        d_hsScale.set_draw_value (TRUE);
	d_hbox1.pack_start (d_hsScale, TRUE, TRUE, s_HBOff);
        d_hsScale.show ();
	d_vbox1.pack_start (d_hbox1, TRUE, TRUE, s_VBOff);
	d_hbox1.show ();

	d_msgSync.set_justify (GTK_JUSTIFY_LEFT);
	d_hbox2.pack_start (d_msgSync, TRUE, TRUE, s_HBOff);
	d_msgSync.show ();
        d_hsSync.set_update_policy (GTK_UPDATE_DELAYED);
        d_hsSync.set_digits (0);
        d_hsSync.set_draw_value (TRUE);
	d_hbox2.pack_start (d_hsSync, TRUE, TRUE, s_HBOff);
        d_hsSync.show ();
	d_vbox1.pack_start (d_hbox2, TRUE, TRUE, s_VBOff);
	d_hbox2.show ();

	d_msgFillSea.set_justify (GTK_JUSTIFY_LEFT);
	d_hbox3.pack_start (d_msgFillSea, TRUE, TRUE, s_HBOff);
	d_msgFillSea.show ();
	d_cbFillSea.set_active (TRUE); 
	d_hbox3.pack_start (d_cbFillSea, TRUE, TRUE, s_HBOff);
	d_cbFillSea.show ();
	d_vbox1.pack_start (d_hbox3, TRUE, TRUE, s_VBOff);
	d_hbox3.show ();

	d_frame1.add (d_vbox1);
	d_vbox1.show ();
	d_frame1.show ();
}


/*
 *  buildHFDrawOptionsFrame: frame for the Height Field draw options
 */
void TFDialogOptions::buildHFDrawOptionsFrame ()
{
	d_msgResolution.set_justify (GTK_JUSTIFY_LEFT);
	d_hbox4.pack_start (d_msgResolution, TRUE, TRUE, s_HBOff);
	d_msgResolution.show ();
        d_hsResolution.set_update_policy (GTK_UPDATE_DELAYED);
        d_hsResolution.set_digits (0);
        d_hsResolution.set_draw_value (TRUE);
	d_hbox4.pack_start (d_hsResolution, TRUE, TRUE, s_HBOff);
        d_hsResolution.show ();
	d_vbox2.pack_start (d_hbox4, TRUE, TRUE, s_VBOff);
	d_hbox4.show ();

	d_msgYScale.set_justify (GTK_JUSTIFY_LEFT);
	d_hbox5.pack_start (d_msgYScale, TRUE, TRUE, s_HBOff);
	d_msgYScale.show ();
        d_hsYScale.set_update_policy (GTK_UPDATE_DELAYED);
        d_hsYScale.set_digits (2);
        d_hsYScale.set_draw_value (TRUE);
	d_hbox5.pack_start (d_hsYScale, TRUE, TRUE, s_HBOff);
        d_hsYScale.show ();
	d_vbox2.pack_start (d_hbox5, TRUE, TRUE, s_VBOff);
	d_hbox5.show ();

	d_msgSealevel.set_justify (GTK_JUSTIFY_LEFT);
	d_hbox6.pack_start (d_msgSealevel, TRUE, TRUE, s_HBOff);
	d_msgSealevel.show ();
        d_hsSealevel.set_update_policy (GTK_UPDATE_DELAYED);
        d_hsSealevel.set_digits (2);
        d_hsSealevel.set_draw_value (TRUE);
	d_hbox6.pack_start (d_hsSealevel, TRUE, TRUE, s_HBOff);
        d_hsSealevel.show ();
	d_vbox2.pack_start (d_hbox6, TRUE, TRUE, s_VBOff);
	d_hbox6.show ();

	d_frame2.add (&d_vbox2);
	d_vbox2.show ();
	d_frame2.show ();
}

/*
 *  buildFastDrawOptionsFrame: frame for the Height Field draw options
 */
void TFDialogOptions::buildFastWireOptionsFrame ()
{
	d_msgFastResolution.set_justify (GTK_JUSTIFY_LEFT);
	d_hbox7.pack_start (d_msgFastResolution, TRUE, TRUE, s_HBOff);
	d_msgFastResolution.show ();
	d_cbFastResolution.set_active (TRUE);
	d_hbox7.pack_start (d_cbFastResolution, TRUE, TRUE, s_HBOff);
	d_cbFastResolution.show ();
	d_vbox3.pack_start (d_hbox7, TRUE, TRUE, s_VBOff);
	d_hbox7.show ();

	d_msgFastYscale.set_justify (GTK_JUSTIFY_LEFT);
	d_hbox8.pack_start (d_msgFastYscale, TRUE, TRUE, s_HBOff);
	d_msgFastYscale.show ();
	d_cbFastYscale.set_active (TRUE);
	d_hbox8.pack_start (d_cbFastYscale, TRUE, TRUE, s_HBOff);
	d_cbFastYscale.show ();
	d_vbox3.pack_start (d_hbox8, TRUE, TRUE, s_VBOff);
	d_hbox8.show ();

	d_msgFastZrot.set_justify (GTK_JUSTIFY_LEFT);
	d_hbox9.pack_start (d_msgFastZrot, TRUE, TRUE, s_HBOff);
	d_msgFastZrot.show ();
	d_cbFastZrot.set_active (FALSE);
	d_hbox9.pack_start (d_cbFastZrot, TRUE, TRUE, s_HBOff);
	d_cbFastZrot.show ();
	d_vbox3.pack_start (d_hbox9, TRUE, TRUE, s_VBOff);
	d_hbox9.show ();

	d_frame3.add (&d_vbox3);
	d_vbox3.show ();
	d_frame3.show ();

}


/*
 *  getValues: get all the values from the dialog inputs
 */
void TFDialogOptions::getValues (bool *fillSea, int *scale, int *syncRate,
				 int *resolution, float *yscale, float *sealevel, 
				 bool *fastHalfRes, bool *fastHalfYscale, 
				 bool *fastZrot)
{
	*fillSea = d_cbFillSea.get_active ();

	*scale = (int)(d_adjScale.get_value ()); 

	*syncRate = (int)(d_adjSync.get_value());

	*resolution = (int)(d_adjResolution.get_value ()); 

	*yscale= d_adjYScale.get_value (); 

	*sealevel = d_adjSealevel.get_value (); 

	*fastHalfRes = d_cbFastResolution.get_active ();
	*fastHalfYscale = d_cbFastYscale.get_active ();
	*fastZrot = d_cbFastZrot.get_active ();
}


void TFDialogOptions::buttonCallbackOK ()
{	
	this->hide ();
	gdk_flush ();
	buttonCallbackApply ();
}


/*
 *  buttonCallbackApply: update what's been changed and redraw if necessary
 */
void TFDialogOptions::buttonCallbackApply ()
{
	bool			redraw=FALSE;

	int			scale, 
				syncRate;
	bool			fillSea; 

	int			resolution;
	float			yscale, 
				sealevel;

	getValues (&fillSea, &scale, &syncRate, &resolution, &yscale, &sealevel,
		   &d_fastHalfRes, &d_fastHalfYscale, &d_fastZrot);

	if (fillSea != d_fillSea)
		{
		d_HFD->setSeaFill (fillSea);
		d_fillSea = fillSea;
		redraw = TRUE;
		}

	if (scale != d_scale)	
		{
		d_HFD->setScale (scale);
		d_scale = scale;
		redraw = TRUE;
		}

	if (syncRate != d_syncRate)
		{
		d_HFD->setSync (syncRate);
		d_syncRate = syncRate;
		redraw = TRUE;
		}

	if (resolution != d_resolution)
		{
		d_HFD->setResolution (resolution);
		d_resolution = resolution;
		redraw = TRUE;
		}

	if (yscale != d_yscale)
		{
		d_HFD->setYScale ((double)yscale);
		d_yscale = yscale;
		redraw = TRUE;
		}

	if (sealevel != d_sealevel)
		{
		d_HF->setSealevel (sealevel);
		d_sealevel = sealevel;
		redraw = TRUE;
		}

	d_HFD->setFastOptions (d_fastHalfRes, d_fastHalfYscale, d_fastZrot);

	if (redraw)
		d_HFD->draw ();
}


gint TFDialogOptions::delete_event_impl (GdkEventAny *)
{
	this->hide ();
	return (0);
}




