/* Terraform - (C) 1997-1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "TFDialogPPov.h"
#include "GlobalSanityCheck.h"
#include "GlobalTrace.h"


/*
 *  constructor: initialize all data members 
 */
TFDialogPPov::TFDialogPPov (HeightField *HF)
	        : GuiDialogOAC ("Terraform Place POV Object Dialog", 
				"POV object placement parameters"), 
		  d_vBoxList (TRUE, 5),
		  d_vBox (FALSE, 5),
		  d_hBox1 (FALSE, 5), d_hBox2 (FALSE, 5), d_hBox3 (FALSE, 5),
		  d_hBox4 (FALSE, 5), d_hBox5 (FALSE, 5), d_hBox6 (FALSE, 5),
		  d_hBox7 (FALSE, 5), 
		  //d_lblLine1 ("Output Options"),
		  d_lblLine2 ("Elevation"),
		  d_lblLine3 ("Placement"),
		  d_lblLine4 ("HF Scale"),
		  d_lblLine5 ("Obj Scale"),
		  d_lblLine6 ("Scale Variance"),
		  d_lblLine7 (""),
		  d_lblOutputFilename ("Filename"),
		  d_lblObjectName ("POV Object Name"),
		  d_lblHFScaleX ("X"), 
		  d_lblHFScaleY ("Y"),
		  d_lblHFScaleZ ("Z"),
		  d_lblObjMinElv ("Min"), 
		  d_lblObjMaxElv ("Max"),
		  d_lblPlaceDensity ("Density"), 
		  d_lblPlaceVariance ("Variance"),
		  d_lblObjScaleX ("X"), 
		  d_lblObjScaleY ("Y"), 
		  d_lblObjScaleZ ("Z"),
		  d_lblObjScaleXvar ("X"), 
		  d_lblObjScaleYvar ("Y"), 
		  d_lblObjScaleZvar ("Z"),
		  d_adjHFScaleX (1000, 100, 10000, 1), 
		  d_adjHFScaleY (250, 50, 5000, 1), 
		  d_adjHFScaleZ (1000, 100, 10000, 1),
		  d_adjObjMinElv (0.66, 0.0, 1.0, 0.01), 
		  d_adjObjMaxElv (0.95, 0.0, 1.0, 0.01),
		  d_adjPlaceDensity (0.5, 0.0, 1.0, 0.01), 
		  d_adjPlaceVariance (0.5, 0.0, 1.0, 0.01),
		  d_adjObjScaleX (5, 1, 25, 1), 
		  d_adjObjScaleY (20, 1, 250, 1),
		  d_adjObjScaleZ (5, 1, 25, 1),
		  d_adjObjScaleXvar (0.33, 0.0, 1.0, 0.01), 
		  d_adjObjScaleYvar (0.33, 0.0, 1.0, 0.01), 
		  d_adjObjScaleZvar (0.33, 0.0, 1.0, 0.01),
		  d_hsHFScaleX (d_adjHFScaleX),
		  d_hsHFScaleY (d_adjHFScaleY),
		  d_hsHFScaleZ (d_adjHFScaleZ),
		  d_hsObjMinElv (d_adjObjMinElv),
		  d_hsObjMaxElv (d_adjObjMaxElv),
		  d_hsPlaceDensity (d_adjPlaceDensity),
		  d_hsPlaceVariance (d_adjPlaceVariance),
		  d_hsObjScaleX (d_adjObjScaleX),
		  d_hsObjScaleY (d_adjObjScaleY),
		  d_hsObjScaleZ (d_adjObjScaleZ),
		  d_hsObjScaleXvar (d_adjObjScaleXvar),
		  d_hsObjScaleYvar (d_adjObjScaleYvar),
		  d_hsObjScaleZvar (d_adjObjScaleZvar)
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFDialogPPov\n");

	SanityCheck::bailout ((!HF), "HF==NULL", "TFDialogPPov::TFDialogPPov");

	d_HFO = NULL;
	setHF (HF);
	
	buildDialogWindow ();
}


/*
 *  destructor: clean up 
 */
TFDialogPPov::~TFDialogPPov ()
{
	if (d_HFO)
		delete d_HFO;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFDialogPPov\n");
}


/*
 *  setHFobjs: set the HF objects we operate on
 */
void TFDialogPPov::setHF (HeightField *HF)
{
	SanityCheck::bailout ((!HF), "HF==NULL", "TFDialogPPov::setHFobjs");

	if (d_HFO)
                delete d_HFO;

	d_HF = HF;
	d_HFO = new HeightFieldOps (d_HF);
}


/*
 *  fillVBox: fill the dialog's VBox
 */
void TFDialogPPov::fillVBox ()
{
	int 	s_HBOff = 1,	// hbox offset
		expand=TRUE, 
		fill=FALSE;

	SanityCheck::bailout ((!this->d_baseFrame), "TFDialogPPov called without Frame label",
		"TFDialogPPov::setHF");

	//d_hBox1.pack_start (&d_lblLine1, expand, TRUE, s_HBOff);
	//d_lblLine1.show ();
	d_hBox1.pack_start (d_lblOutputFilename, expand, fill, s_HBOff);
	d_lblOutputFilename.show ();
	d_hBox1.pack_start (d_enOutputFilename, expand, fill+1, s_HBOff);
	d_enOutputFilename.set_text ("hfpovobj.inc");
	d_enOutputFilename.show ();
	d_hBox1.pack_start (d_lblObjectName, expand, fill, s_HBOff);
	d_lblObjectName.show ();
	d_hBox1.pack_start (d_enObjectName, expand, fill+1, s_HBOff);
	d_enObjectName.set_text ("pov_object");
	d_enObjectName.show ();
	d_vBoxList.pack_start (d_hBox1, TRUE, TRUE, s_VBOff);
	d_hBox1.show ();

	d_hBox2.pack_start (d_lblLine2, expand, TRUE, s_HBOff);
	d_lblLine2.show ();
	d_hBox2.pack_start (d_lblObjMinElv, expand, fill, s_HBOff);
	d_lblObjMinElv.show ();
	d_hBox2.pack_start (d_hsObjMinElv, expand, fill+1, s_HBOff);
	d_hsObjMinElv.set_digits (2);
	d_hsObjMinElv.show ();
	d_hBox2.pack_start (d_lblObjMaxElv, expand, fill, s_HBOff);
	d_lblObjMaxElv.show ();
	d_hBox2.pack_start (d_hsObjMaxElv, expand, fill+1, s_HBOff);
	d_hsObjMaxElv.set_digits (2);
	d_hsObjMaxElv.show ();
	d_vBoxList.pack_start (d_hBox2, TRUE, TRUE, s_VBOff);
	d_hBox2.show ();

	d_hBox3.pack_start (d_lblLine3, expand, TRUE, s_HBOff);
	d_lblLine3.show ();
	d_hBox3.pack_start (d_lblPlaceDensity, expand, fill, s_HBOff);
	d_lblPlaceDensity.show ();
	d_hBox3.pack_start (d_hsPlaceDensity, expand, fill+1, s_HBOff);
	d_hsPlaceDensity.set_digits (2);
	d_hsPlaceDensity.show ();
	d_hBox3.pack_start (d_lblPlaceVariance, expand, fill, s_HBOff);
	d_hsPlaceVariance.set_digits (2);
	d_lblPlaceVariance.show ();
	d_hBox3.pack_start (d_hsPlaceVariance, expand, fill+1, s_HBOff);
	d_hsPlaceVariance.show ();
	d_vBoxList.pack_start (d_hBox3, TRUE, TRUE, s_VBOff);
	d_hBox3.show ();

	d_hBox4.pack_start (d_lblLine4, expand, TRUE, s_HBOff);
	d_lblLine4.show ();
	d_hBox4.pack_start (d_lblHFScaleX, expand, fill, s_HBOff);
	d_lblHFScaleX.show ();
	d_hBox4.pack_start (d_hsHFScaleX, expand, fill+1, s_HBOff);
	d_hsHFScaleX.set_digits (0);
	d_hsHFScaleX.show ();
	d_hBox4.pack_start (d_lblHFScaleY, expand, fill, s_HBOff);
	d_hsHFScaleY.set_digits (0);
	d_lblHFScaleY.show ();
	d_hBox4.pack_start (d_hsHFScaleY, expand, fill+1, s_HBOff);
	d_hsHFScaleY.show ();
	d_hBox4.pack_start (d_lblHFScaleZ, expand, fill, s_HBOff);
	d_hsHFScaleZ.set_digits (0);
	d_lblHFScaleZ.show ();
	d_hBox4.pack_start (d_hsHFScaleZ, expand, fill+1, s_HBOff);
	d_hsHFScaleZ.show ();
	d_vBoxList.pack_start (d_hBox4, TRUE, TRUE, s_VBOff);
	d_hBox4.show ();

	d_hBox5.pack_start (d_lblLine5, expand, TRUE, s_HBOff);
	d_lblLine5.show ();
	d_hBox5.pack_start (d_lblObjScaleX, expand, fill, s_HBOff);
	d_lblObjScaleX.show ();
	d_hBox5.pack_start (d_hsObjScaleX, expand, fill+1, s_HBOff);
	d_hsObjScaleX.set_digits (0);
	d_hsObjScaleX.show ();
	d_hBox5.pack_start (d_lblObjScaleY, expand, fill, s_HBOff);
	d_lblObjScaleY.show ();
	d_hsObjScaleY.set_digits (0);
	d_hBox5.pack_start (d_hsObjScaleY, expand, fill+1, s_HBOff);
	d_hsObjScaleY.show ();
	d_hBox5.pack_start (d_lblObjScaleZ, expand, fill, s_HBOff);
	d_hsObjScaleZ.set_digits (0);
	d_lblObjScaleZ.show ();
	d_hBox5.pack_start (d_hsObjScaleZ, expand, fill+1, s_HBOff);
	d_hsObjScaleZ.show ();
	d_vBoxList.pack_start (d_hBox5, TRUE, TRUE, s_VBOff);
	d_hBox5.show ();

	d_hBox6.pack_start (d_lblLine6, expand, TRUE, s_HBOff);
	d_lblLine6.show ();
	d_hBox6.pack_start (d_lblObjScaleXvar, expand, fill, s_HBOff);
	d_lblObjScaleXvar.show ();
	d_hBox6.pack_start (d_hsObjScaleXvar, expand, fill+1, s_HBOff);
	d_hsObjScaleXvar.show ();
	d_hBox6.pack_start (d_lblObjScaleYvar, expand, fill, s_HBOff);
	d_lblObjScaleYvar.show ();
	d_hBox6.pack_start (d_hsObjScaleYvar, expand, fill+1, s_HBOff);
	d_hsObjScaleYvar.show ();
	d_hBox6.pack_start (d_lblObjScaleZvar, expand, fill, s_HBOff);
	d_lblObjScaleZvar.show ();
	d_hBox6.pack_start (d_hsObjScaleZvar, expand, fill+1, s_HBOff);
	d_hsObjScaleZvar.show ();
	d_vBoxList.pack_start (d_hBox6, TRUE, TRUE, s_VBOff);
	d_hBox6.show ();

	this->d_baseFrame->add (d_vBoxList);
	d_vBoxList.show ();
	this->d_baseFrame->show ();
	this->get_vbox()->show ();
}



void TFDialogPPov::buildDialogWindow ()
{
	this->set_usize (500, 300);
	this->set_title (d_winTitle);

	this->get_vbox()->set_border_width (2);

	this->setupVBox ();
	fillVBox ();
	this->fillActionArea ();
	
	this->show ();
}


void TFDialogPPov::buttonCallbackOK ()
{	
	this->hide ();
	gdk_flush ();
	buttonCallbackApply ();
}


/*
 *  buttonCallbackApply: transform the height field
 */
void TFDialogPPov::buttonCallbackApply ()
{
	char 		*outfile, 
			*object;
	int		hfsx, hfsy, hfsz, 
			osx, osy, osz;
	float		floor, ceil,
			density, pvar, 
			osxvar, osyvar, oszvar;

	outfile = const_cast<char*>(d_enOutputFilename.get_text().data());
	// this alters both char* strings (why??)
	//object = const_cast<char*> (d_enObjectName.get_text().data());	// this alters both char* strings (why??)
	// XXXXXXXXX
	object = (char *) "pov_object";
	hfsx = (int)(d_adjHFScaleX.get_value());
	hfsy = (int)(d_adjHFScaleY.get_value());
	hfsz = (int)(d_adjHFScaleZ.get_value());
	floor = d_adjObjMinElv.get_value();
	ceil = d_adjObjMaxElv.get_value();
	density = d_adjPlaceDensity.get_value();
	pvar = d_adjPlaceVariance.get_value();
	osx = (int)(d_adjObjScaleX.get_value());
	osy = (int)(d_adjObjScaleY.get_value());
	osz = (int)(d_adjObjScaleZ.get_value());
	osxvar = d_adjObjScaleXvar.get_value();
	osyvar = d_adjObjScaleYvar.get_value();
	oszvar = d_adjObjScaleZvar.get_value();

	cout << "Names: " << outfile << ", " << object << "\n"; 
	//exit (0);

	SanityCheck::bailout ((!d_HFO), "d_HFO==NULL", "TFDialogRotate::buttonCallbackApply");
	d_HFO->placePOVObject (outfile, object, 
			hfsx, hfsy, hfsz, 
			floor, ceil, density, pvar,
			osx, osy, osz, osxvar, osyvar, oszvar, 0);
}


gint TFDialogPPov::delete_event_impl (GdkEventAny *)
{
	this->hide ();
	return (0);
}


/*
int GlobalTrace::d_traceLevel=255;    // instantiate static data member
int main (int argc, char *argv[])
{
	Gtk_Main        gtkMain (&argc, &argv);
	TFDialogPPov *testDialog;

	testDialog = new TFDialogPPov (NULL, NULL, "TestTitle", "TestMessage");
	testDialog->show ();
	gtkMain.run ();
}
*/
