/* Terraform - (C) 1997-1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "TFDialogRescale.h"
#include "GlobalSanityCheck.h"
#include "GlobalTrace.h"


/*
 *  constructor: initialize all data members 
 */
TFDialogRescale::TFDialogRescale (HeightField *HF, HeightFieldDraw *HFD)
	        : GuiDialogOAC ("Terraform Rescale Dialog", 
				"Operation"),
		  d_vBoxList (TRUE, 5),
		  d_hbFLocal (TRUE, 5),
		  d_hbFGlobal (TRUE, 5),
		  d_adjFLocal (0, 0, 1, 0.01), 
		  d_adjFGlobal (1, 0, 1, 0.01), 
		  d_hsFLocal (d_adjFLocal),
		  d_hsFGlobal (d_adjFGlobal),
		  d_lblFLocal ("Local fractal scaling"),
		  d_lblFGlobal ("Glocal fractal scaling")
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFDialogRescale\n");

	d_HFO = NULL;
	setHFobjs (HF, HFD);
	d_RB1 = d_RB2 = NULL;
	d_group = NULL;

	buildDialogWindow ();
}


/*
 *  destructor: clean up 
 */
TFDialogRescale::~TFDialogRescale ()
{
	if (d_HFO) delete d_HFO;
	if (d_RB1) delete d_RB1;
	if (d_RB2) delete d_RB2;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFDialogRescale\n");
}


/*
 *  setHBobjs: set the HB objects we operate on 
 */
void TFDialogRescale::setHFobjs (HeightField *HF, HeightFieldDraw *HFD)
{
	SanityCheck::bailout ((!HF), "HF==NULL", "TFDialogRescale::setHFobjs");
	SanityCheck::bailout ((!HFD), "HFD==NULL", "TFDialogRescale::setHFobjs");

	if (HF==d_HF && HFD==d_HFD)
		return;

	if (d_HFO) delete d_HFO;

	d_HF = HF;
	d_HFD = HFD;
	d_HFO = new HeightFieldOps (d_HF);
}


/*
 *  fillVBox: fill the dialog's VBox
 */
void TFDialogRescale::fillVBox ()
{
	SanityCheck::bailout ((!this->d_baseFrame), "TFDialogRescale called without Frame label", 
		"TFDialogRescale::setHFobjs");

	d_RB1 = new Gtk_RadioButton ((GSList *)NULL, "Half Size");
	connect_to_method (d_RB1->clicked, this, &rbCallback);
	d_RB1->set_active (FALSE);
	d_vBoxList.pack_start (*d_RB1, TRUE, TRUE, 0);
	d_RB1->show ();

	d_group = d_RB1->group ();
	d_RB2 = new Gtk_RadioButton (d_group, "Double Size");
	d_RB2->set_active (TRUE);
	connect_to_method (d_RB2->clicked, this, &rbCallback);
	d_vBoxList.pack_start (*d_RB2, TRUE, TRUE, s_VBOff);
	d_RB2->show ();

	d_hbFLocal.pack_start (d_lblFLocal, TRUE, TRUE, s_HBOff);
	d_lblFLocal.show ();
	d_hsFGlobal.set_digits (2);
	d_hsFGlobal.set_draw_value (TRUE);
	d_hbFLocal.pack_end (d_hsFLocal, TRUE, TRUE, s_HBOff);
	d_hsFLocal.show ();
	d_vBoxList.pack_start (d_hbFLocal, TRUE, TRUE, s_VBOff);
	d_hbFLocal.show ();

	d_hbFGlobal.pack_start (d_lblFGlobal, TRUE, TRUE, s_HBOff);
	d_lblFGlobal.show ();
	d_hsFLocal.set_digits (2);
	d_hsFLocal.set_draw_value (TRUE);
	d_hbFGlobal.pack_end (d_hsFGlobal, TRUE, TRUE, s_HBOff);
	d_hsFGlobal.show ();
	d_vBoxList.pack_start (d_hbFGlobal, TRUE, TRUE, s_VBOff);
	d_hbFGlobal.show ();

	this->d_baseFrame->add (d_vBoxList);

	d_vBoxList.show ();
	this->d_baseFrame->show ();
	this->get_vbox()->show ();
}


/*
 *  buildDialogWindow:
 */
void TFDialogRescale::buildDialogWindow ()
{
	this->set_usize (300, 200);
	this->set_title (d_winTitle);

	this->get_vbox()->set_border_width (2);

	this->setupVBox ();
	fillVBox ();
	fillActionArea ();
	rbCallback ();

	this->show ();
}


void TFDialogRescale::buttonCallbackOK ()
{	
	this->hide ();
	gdk_flush ();
	buttonCallbackApply ();
}


/*
 *  buttonCallbackApply: rotate the Height Field
 */
void TFDialogRescale::buttonCallbackApply ()
{
	SanityCheck::bailout ((!d_HFO), "d_HFO==NULL", "TFDialogRescale::buttonCallbackApply");

	if (d_RB1->get_active())
		{
		d_HFO->hfHalf ();
		d_HFD->setScale (1);
		}
	else
	if (d_RB2->get_active())
		{
		float	v1, v2;

		v1 = d_adjFLocal.get_value ();
		v2 = d_adjFGlobal.get_value ();
		d_HFO->hfDouble ((double)v1, (double)v2);
		d_HFD->setScale (1);
		}

	d_HFD->draw ();
}


/*
 *  rbCallback: callback for the radio button HScale on/off toggle
 */
void TFDialogRescale::rbCallback ()
{
	if (d_RB1->get_active())
		{
		d_lblFLocal.set_sensitive (FALSE);
		d_lblFGlobal.set_sensitive (FALSE);
		d_hsFLocal.set_sensitive (FALSE);
		d_hsFGlobal.set_sensitive (FALSE);
		}
	else
		{
		d_lblFLocal.set_sensitive (TRUE);
		d_lblFGlobal.set_sensitive (TRUE);
		d_hsFLocal.set_sensitive (TRUE);
		d_hsFGlobal.set_sensitive (TRUE);
		}
}


gint TFDialogRescale::delete_event_impl (GdkEventAny *)
{
	this->hide ();
	return (0);
}


/*
int GlobalTrace::d_traceLevel=255;    // instantiate static data member
int main (int argc, char *argv[])
{
	Gtk_Main        gtkMain (&argc, &argv);
	TFDialogRescale *testDialog;

	testDialog = new TFDialogRescale (NULL, NULL, "TestTitle", "TestMessage");
	testDialog->show ();
	gtkMain.run ();
}
*/
