/* Terraform - (C) 1997-1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "TFDialogRotate.h"
#include "HeightFieldOps.h"
#include "GlobalSanityCheck.h"
#include "GlobalTrace.h"


/*
 *  constructor: initialize all data members 
 */
TFDialogRotate::TFDialogRotate (HeightField *HF, HeightFieldDraw *HFD)
	        : GuiDialogOAC ("Terraform Rotate Dialog", NULL),
		d_frame ("Rotate clockwise by"),
		d_table (6, 6, FALSE),
		d_rLabel1 ("0"),
		d_rLabel2 ("90"),
		d_rLabel3 ("180"),
		d_rLabel4 ("270"),
		d_unitLabel ("degrees")
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFDialogRotate\n");

	d_HFO = NULL;
	setHFobjs (HF, HFD);

	buildDialogWindow ();
}


/*
 *  destructor: clean up 
 */
TFDialogRotate::~TFDialogRotate ()
{
	if (d_HFO) 	delete d_HFO;
	if (d_rButton1) delete d_rButton1;
	if (d_rButton2) delete d_rButton2;
	if (d_rButton3) delete d_rButton3;
	if (d_rButton4) delete d_rButton4;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFDialogRotate\n");
}


/*
 *  setHBobjs: set the HB objects we operate on 
 */
void TFDialogRotate::setHFobjs (HeightField *HF, HeightFieldDraw *HFD)
{
	SanityCheck::bailout ((!HF), "HF==NULL", "TFDialogRotate::setHFobjs");
	SanityCheck::bailout ((!HFD), "HFD==NULL", "TFDialogRotate::setHFobjs");

	if (d_HFO)
		delete d_HFO;

	d_HF = HF;
	d_HFD = HFD;
	d_HFO = new HeightFieldOps (d_HF);
}


/*
 *  fillVBox: fill the dialog's VBox
 */
void TFDialogRotate::fillVBox ()
{
	GSList		*radioGroup=NULL;

	// parameter settings  
	d_frame.set_shadow_type (GTK_SHADOW_ETCHED_IN);
	d_frame.set_border_width (6);
	this->get_vbox()->pack_start (d_frame, TRUE, TRUE, 0);

	// table for d_rButtons 
	d_table.set_row_spacings (8);
	d_table.set_col_spacings (1);
	d_table.set_border_width (6);
	d_frame.add (d_table);

	// place the buttons 
	d_table.attach (d_rLabel1, 2, 3 , 0, 1, 0, 0, 0, 0);
	d_rLabel1.show ();
	d_rButton1 = new Gtk_RadioButton (radioGroup);
	radioGroup = d_rButton1->group();
	d_table.attach (*d_rButton1, 2, 3, 1, 2, 0, 0, 0, 0);
	d_rButton1->show ();

	d_table.attach (d_rLabel2, 4, 5, 2, 3, 0, 0, 0, 0);
	d_rLabel2.show ();
	d_rButton2 = new Gtk_RadioButton (radioGroup);
	radioGroup = d_rButton2->group();
	d_table.attach (*d_rButton2, 3, 4, 2, 3, 0, 0, 0, 0);
	d_rButton2->show ();

	d_table.attach (d_rLabel3, 2, 3, 4, 5, 0, 0, 0, 0);
	d_rLabel3.show ();
	d_rButton3 = new Gtk_RadioButton (radioGroup);
	radioGroup = d_rButton3->group();
	d_table.attach (*d_rButton3, 2, 3, 3, 4, 0, 0, 0, 0);
	d_rButton3->set_active (TRUE);
	d_rButton3->show ();

	d_table.attach (d_rLabel4, 0, 1, 2, 3, 0, 0, 0, 0);
	d_rLabel4.show ();
	d_rButton4 = new Gtk_RadioButton (radioGroup);
	radioGroup = d_rButton4->group();
	d_table.attach (*d_rButton4, 1, 2, 2, 3, 0, 0, 0, 0);
	d_rButton4->show ();

	// place unit label
	d_table.attach (d_unitLabel, 5, 6, 5, 6, 0, 0, 0, 0);
	d_unitLabel.show ();

	d_table.show();
	d_frame.show ();
}


/*
 *  buildDialogWindow:
 */
void TFDialogRotate::buildDialogWindow ()
{
	this->set_usize (175, 215);
	this->set_title (d_winTitle);

	this->get_vbox()->set_border_width (2);

	fillVBox ();
	fillActionArea ();

	this->show ();
}


void TFDialogRotate::buttonCallbackOK ()
{	
	this->hide ();
	gdk_flush ();
	buttonCallbackApply ();
}


/*
 *  buttonCallbackApply: rotate the Height Field
 */
void TFDialogRotate::buttonCallbackApply ()
{
	int		value=180; 

	if (d_rButton1->get_active())
		value = 0;
	else
	if (d_rButton2->get_active())
		//value = 90;
		value = 270;		// Rotate90 goes counterclockwise
	else
	if (d_rButton3->get_active())
		value = 180;
	else
	if (d_rButton4->get_active())
		//value = 270;
		value = 90;		// Rotate90 goes counterclockwise
	else
		SanityCheck::bailout ((TRUE), "Button out of range", "TFDialogRotate::buttonCallbackApply");

	SanityCheck::bailout ((!d_HFO), "d_HFO==NULL", "TFDialogRotate::buttonCallbackApply");

	if (value)
		{
		d_HFO->rotate90 (value);
		d_HFD->draw ();
		}
}


gint TFDialogRotate::delete_event_impl (GdkEventAny *)
{
	this->hide ();
	return (0);
}


/*
int GlobalTrace::d_traceLevel=255;    // instantiate static data member
int main (int argc, char *argv[])
{
	Gtk_Main        gtkMain (&argc, &argv);
	TFDialogRotate *testDialog;

	testDialog = new TFDialogRotate (NULL, NULL, "TestTitle", "TestMessage");
	testDialog->show ();
	gtkMain.run ();
}
*/
