/* Terraform - (C) 1997-1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "TFDialogTransform.h"
#include "GlobalSanityCheck.h"
#include "GlobalTrace.h"


/*
 *  constructor: initialize all data members 
 */
TFDialogTransform::TFDialogTransform (HeightField *HF, HeightFieldDraw *HFD)
	        : GuiDialogOAC ("Terraform Transform Dialog", 
				"Transform: set v for h = h^v"), 
		  d_vBoxList (TRUE, 5),
		  d_hBoxTFactor (TRUE, 5),
		  d_hBoxInvert (TRUE, 5),
		  d_hBoxAdjustSea (TRUE, 5),
		  d_adjTFactor (1.5, 1.0, 5.0, 0.01),
		  d_hsTFactor (d_adjTFactor),
		  d_lblTFactor ("Power Factor "),
		  d_lblInvert ("Invert Power Factor"),
		  d_lblAdjustSea ("Adjust Waterlevel")

{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFDialogTransform\n");

	d_HFO = NULL;
	setHFobjs (HF, HFD);
	
	buildDialogWindow ();
}


/*
 *  destructor: clean up 
 */
TFDialogTransform::~TFDialogTransform ()
{
	if (d_HFO)
		delete d_HFO;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFDialogTransform\n");
}


/*
 *  setHFobjs: set the HF objects we operate on
 */
void TFDialogTransform::setHFobjs (HeightField *HF, HeightFieldDraw *HFD)
{
	SanityCheck::bailout ((!HF), "HF==NULL", "TFDialogTransform::setHFobjs");
	SanityCheck::bailout ((!HFD), "HFD==NULL", "TFDialogTransform::setHFobjs");

	if (d_HFO)
		delete d_HFO;

	d_HF = HF;
	d_HFD = HFD;
	d_HFO = new HeightFieldOps (d_HF);
}


/*
 *  fillVBox: fill the dialog's VBox
 */
void TFDialogTransform::fillVBox ()
{
	SanityCheck::bailout ((!this->d_baseFrame), "TFDialogTransform called without Frame label",
		"TFDialogTransform ::setHFobjs");

	d_hBoxTFactor.pack_start (d_lblTFactor, TRUE, TRUE, s_HBOff);
	d_lblTFactor.show ();
	d_hsTFactor.set_update_policy (GTK_UPDATE_DELAYED);
	d_hsTFactor.set_digits (2);
	d_hsTFactor.set_draw_value (TRUE);
	d_hBoxTFactor.pack_end (d_hsTFactor, TRUE, TRUE, s_HBOff);
	d_hsTFactor.show ();
	d_vBoxList.pack_start (d_hBoxTFactor, TRUE, TRUE, s_VBOff);
	d_hBoxTFactor.show ();

	d_hBoxInvert.pack_start (d_lblInvert, TRUE, TRUE, s_HBOff);
	d_lblInvert.show ();
	d_btnInvert.set_active (FALSE);
	connect_to_method (d_btnInvert.clicked, this, &invertCBCallback);
	d_hBoxInvert.pack_end (d_btnInvert, TRUE, TRUE, s_HBOff);
	d_btnInvert.show ();
	d_vBoxList.pack_start (d_hBoxInvert, TRUE, TRUE, s_VBOff);
	d_hBoxInvert.show ();

	d_hBoxAdjustSea.pack_start (d_lblAdjustSea, TRUE, TRUE, s_HBOff);
	d_lblAdjustSea.show ();
	d_btnAdjustSea.set_active (TRUE);
	d_hBoxAdjustSea.pack_end (d_btnAdjustSea, TRUE, TRUE, s_HBOff);
	d_btnAdjustSea.show ();
	d_vBoxList.pack_start (d_hBoxAdjustSea, TRUE, TRUE, s_VBOff);
	d_hBoxAdjustSea.show ();

	this->d_baseFrame->add (d_vBoxList);
	d_vBoxList.show ();
	this->d_baseFrame->show ();
	this->get_vbox()->show ();

}



void TFDialogTransform::buildDialogWindow ()
{
	this->set_usize (250, 175);
	this->set_title (this->d_winTitle);

	this->get_vbox()->set_border_width (2);

	this->setupVBox ();
	fillVBox ();
	this->fillActionArea ();
	
	this->show ();
}


void TFDialogTransform::buttonCallbackOK ()
{	
	this->hide ();
	gdk_flush ();
	buttonCallbackApply ();
}


void TFDialogTransform::invertCBCallback ()
{	
	d_baseFrame->set_label ();
	if (d_btnInvert.get_active())
		d_baseFrame->set_label ("Transform: set v for h = h^(1/v)"); 
	else
		d_baseFrame->set_label ("Transform: set v for h = h^v"); 
}



/*
 *  buttonCallbackApply: transform the height field
 */
void TFDialogTransform::buttonCallbackApply ()
{
	bool			invert=FALSE;
	float			value; 

	value = d_adjTFactor.get_value (); 
	invert = d_btnInvert.get_active ();

	SanityCheck::bailout ((!d_HFO), "d_HFO==NULL", "TFDialogTransform::buttonCallbackApply");

	if (invert)
		d_HFO->transform ((double)1/(double)value, d_btnAdjustSea.get_active());
	else
		d_HFO->transform ((double)value, d_btnAdjustSea.get_active());

	d_HFD->draw ();
}


gint TFDialogTransform::delete_event_impl (GdkEventAny *)
{
	this->hide ();
	return (0);
}


/*
int GlobalTrace::d_traceLevel=255;    // instantiate static data member
int main (int argc, char *argv[])
{
	Gtk_Main        gtkMain (&argc, &argv);
	TFDialogTransform *testDialog;

	testDialog = new TFDialogTransform (NULL, NULL, "TestTitle", "TestMessage");
	testDialog->show ();
	gtkMain.run ();
}
*/
