/* Terraform - (C) 1997-1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include <string.h>
#include "TFWindow.h"
#include "HeightField.h"
#include "GlobalTrace.h"
#include "GlobalSanityCheck.h"


// base color for each color band 
static char	*bandBaseCol[]=	
			{	
				"MidnightBlue",
				"MediumBlue",
				"Blue",
				"Dodger Blue",
				"Green2",
				"Green3",
				"Green4",
				"DarkOliveGreen4",
				"LightGoldenrod2",
				"LightGoldenrod3",
				"LightGoldenrod4",
				"LightYellow4",
				"DimGray",
				"Gray",
				"LightGray",	};

static char 	*bandBaseCol_Desert[]=
			{
				"LightGoldenrod3",
				"LightGoldenrod4",
				"DimGray",
				"LightSlateGray",
				"Gray",
				"LightGray",	};
				

guint TFWhash (gconstpointer key);			// return hash for key
gint TFWhashcmp (gconstpointer a, gconstpointer b);	// compare 

 
/*
 *  constructor: build the window with an itemfactory, buffered drawing area 
 * 	and colormaps. Store the generated menu in a hash table so we have 
 *	a handy reference later we can use for set_sensitvity calls
 */
TFWindow::TFWindow() : GuiBufferedDrawingArea (),
			d_win (GTK_WINDOW_TOPLEVEL),
       			d_vbox (FALSE, 0),
			d_accelGroup ()
	{
	char			*s;
	GtkStyle		*style; 
	Gtk_Style		Style; 

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFWindow\n");

	d_itemFactory = new Gtk_ItemFactory_MenuBar("<Main>", d_accelGroup);
	d_win.add (d_vbox);
	style = Style.gtkobj();
	gdk_font_unref (style->font);
	style->font = gdk_font_load ("-*-Helvetica-Medium-R-Normal--*-100-*-*-*-*-*-*");

    	//d_menuBarHandle = static_cast<Gtk_ObjectHandle<Gtk_MenuBar>&> (d_itemFactory->get_menubar_widget(""));
	{
	// uglier than above but without compiler warning 
	Gtk_ObjectHandle<Gtk_MenuBar> a(d_itemFactory->get_menubar_widget(""));
    	d_menuBarHandle = a;
	}

	SanityCheck::bailout ((!d_menuBarHandle.get_object()), "d_menuBarHandle.get_object failed", "TFWindow::TFWindow");
	d_menuBarHandle->push_style (Style);

	d_menuHash = new Glib_Hash (TFWhash, TFWhashcmp);
	d_mNames = new char*[50];
	s = d_mNames[d_nameCount++] = strdup (MENU_FILE_BRANCH);
	d_menuHash->insert (s, d_itemFactory->create_item (s, 0, "<Branch>", 0));

	s = d_mNames[d_nameCount++] = strdup (MENU_FILE_NEW_BRANCH);
	d_menuHash->insert (s, d_itemFactory->create_item (s, 0, "<Branch>", 0));
	s = d_mNames[d_nameCount++] = strdup (MENU_FILE_NEW_SYNTHESIS);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_FILE_NEW_SYNTHESIS_AC, "", 
		ItemFactoryConnector<TFWindow, string>(this,&fileMenuCallback, 
			s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_FILE_NEW_SUBDIVIDE);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_FILE_NEW_SUBDIVIDE_AC, "", 
		ItemFactoryConnector<TFWindow, string>(this,&fileMenuCallback, 
			s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_FILE_NEW_RANDOM);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_FILE_NEW_RANDOM_AC, "", 
		ItemFactoryConnector<TFWindow, string>(this,&fileMenuCallback, 
			s)));

	s = d_mNames[d_nameCount++] = strdup (MENU_FILE_OPEN);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_FILE_OPEN_AC, "", 
	    ItemFactoryConnector<TFWindow, string>(this,&fileMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_FILE_RELOAD);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_FILE_RELOAD_AC, "",
	    ItemFactoryConnector<TFWindow, string>(this,&fileMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_FILE_SAVE);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_FILE_SAVE_AC, "",
	    ItemFactoryConnector<TFWindow, string>(this,&fileMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_FILE_SAVEAS);
	d_menuHash->insert (s, d_itemFactory->create_item (s ,MENU_FILE_SAVEAS_AC, "",
	    ItemFactoryConnector<TFWindow, string>(this,&fileMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_FILE_CLOSE);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_FILE_CLOSE_AC, "",
	    ItemFactoryConnector<TFWindow, string>(this,&fileMenuCallback, s)));
	d_itemFactory->create_item ("/File/", 		0, "<Separator>", 0);
	s = d_mNames[d_nameCount++] = strdup (MENU_FILE_OPTIONS);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_FILE_OPTIONS_AC, "", 
	    ItemFactoryConnector<TFWindow, string>(this,&fileMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_FILE_ABOUT);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_FILE_ABOUT_AC, "", 
	    ItemFactoryConnector<TFWindow, string>(this,&fileMenuCallback, s)));
	d_itemFactory->create_item ("/File/", 		0, "<Separator>" ,0);
	s = d_mNames[d_nameCount++] = strdup (MENU_FILE_EXIT);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_FILE_EXIT_AC, "",
	    ItemFactoryConnector<TFWindow, string>(this,&fileMenuCallback, s)));

	s = d_mNames[d_nameCount++] = strdup (MENU_HF_BRANCH);
	d_menuHash->insert (s, d_itemFactory->create_item (s, 0, "<Branch>", 0));
	s = d_mNames[d_nameCount++] = strdup (MENU_HF_FILL);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_HF_FILL_AC, "", 
	    ItemFactoryConnector<TFWindow, string>(this,&hfMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_HF_FLOWMAP);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_HF_FLOWMAP_AC, "", 
	    ItemFactoryConnector<TFWindow, string>(this,&hfMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_HF_FOLD);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_HF_FOLD_AC, "",
	    ItemFactoryConnector<TFWindow, string>(this,&hfMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_HF_INVERT);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_HF_INVERT_AC, "",
	    ItemFactoryConnector<TFWindow, string>(this,&hfMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_HF_LINSCALE);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_HF_LINSCALE_AC, "",
	    ItemFactoryConnector<TFWindow, string>(this,&hfMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_HF_MIRROR);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_HF_MIRROR_AC, "",
	    ItemFactoryConnector<TFWindow, string>(this,&hfMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_HF_RESCALE);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_HF_RESCALE_AC, "",
	    ItemFactoryConnector<TFWindow, string>(this,&hfMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_HF_ROTATE);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_HF_ROTATE_AC, "",
	    ItemFactoryConnector<TFWindow, string>(this,&hfMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_HF_ROUGHEN);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_HF_ROUGHEN_AC, "",
	    ItemFactoryConnector<TFWindow, string>(this,&hfMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_HF_SMOOTH);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_HF_SMOOTH_AC, "",
	    ItemFactoryConnector<TFWindow, string>(this,&hfMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_HF_TRANSFORM);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_HF_TRANSFORM_AC, "",
	    ItemFactoryConnector<TFWindow, string>(this,&hfMenuCallback, s)));
	d_itemFactory->create_item ("/HF/", 		0, "<Separator>", 0);
	s = d_mNames[d_nameCount++] = strdup (MENU_HF_ERODE);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_HF_ERODE_AC, "",
	    ItemFactoryConnector<TFWindow, string>(this,&hfMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_HF_PLACEPOV);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_HF_PLACEPOV_AC, "",
	    ItemFactoryConnector<TFWindow, string>(this,&hfMenuCallback, s)));

	s = d_mNames[d_nameCount++] = strdup (MENU_VIEW_BRANCH);
	d_menuHash->insert (s, d_itemFactory->create_item (s, 0, "<Branch>", 0));
	s = d_mNames[d_nameCount++] = strdup (MENU_VIEW_2DPLANE);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_VIEW_2DPLANE_AC, "<RadioItem>",
	    ItemFactoryConnector<TFWindow, string>(this,&viewMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_VIEW_3DWIRE);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_VIEW_3DWIRE_AC, MENU_VIEW_2DPLANE,
	    ItemFactoryConnector<TFWindow, string>(this,&viewMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_VIEW_3DHEIGHT);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_VIEW_3DHEIGHT_AC, MENU_VIEW_2DPLANE,
	    ItemFactoryConnector<TFWindow, string>(this,&viewMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_VIEW_3DLIGHT);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_VIEW_3DLIGHT_AC, MENU_VIEW_2DPLANE,
	    ItemFactoryConnector<TFWindow, string>(this,&viewMenuCallback, s)));
	d_itemFactory->create_item ("/View/", 		0, "<Separator>", 0);
	s = d_mNames[d_nameCount++] = strdup (MENU_VIEW_FASTWIRE);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_VIEW_FASTWIRE_AC, "<ToggleItem>",
	    ItemFactoryConnector<TFWindow, string>(this,&viewMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_VIEW_AUTOROTATE);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_VIEW_AUTOROTATE_AC, "<ToggleItem>",
	    ItemFactoryConnector<TFWindow, string>(this,&viewMenuCallback, s)));

	s = d_mNames[d_nameCount++] = strdup (MENU_CMAP_BRANCH);
	d_menuHash->insert (s, d_itemFactory->create_item (s, 0, "<Branch>", 0));
	s = d_mNames[d_nameCount++] = strdup (MENU_CMAP_COLORBANDS);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_CMAP_COLORBANDS_AC, "<RadioItem>", 
	    ItemFactoryConnector<TFWindow, string>(this,&colormapMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_CMAP_DESERT);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_CMAP_DESERT_AC, MENU_CMAP_COLORBANDS, 
	    ItemFactoryConnector<TFWindow, string>(this,&colormapMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_CMAP_GRAYSCALE);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_CMAP_GRAYSCALE_AC, MENU_CMAP_COLORBANDS,
	    ItemFactoryConnector<TFWindow, string>(this,&colormapMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_CMAP_REDHOT);
	d_menuHash->insert (s, d_itemFactory->create_item (s, MENU_CMAP_REDHOT_AC, MENU_CMAP_COLORBANDS,
	    ItemFactoryConnector<TFWindow, string>(this,&colormapMenuCallback, s)));

	d_accelGroup.attach (d_win);

	d_vbox.pack_start(*d_menuBarHandle, FALSE, FALSE, 0);
	//connect_to_method(ebox.button_press_event, this, &(buttonPress));
	//d_menuBarHandle->show();

	// create and attach drawing area 
	d_vbox.pack_start (*this, TRUE, TRUE, 0);

	// create colormaps
	d_linearColorMap = new GuiColormapLinear (this->get_toplevel(), 255);
	d_linearColorMap->allocColors (225, 0, 0, 0, 1, 1, 1);

	d_linearColorMap2 = new GuiColormapLinear (this->get_toplevel(), 255);
	d_linearColorMap2->allocColors (225, 0, 1, 0, 1, 0, 0);

	d_bandColorMap = new GuiColormapBands (this->get_toplevel(), 15, 15, 4);
	d_bandColorMap->allocColorBands (bandBaseCol); 

	d_bandColorMap2 = new GuiColormapBands (this->get_toplevel(), 6, 15, 1);
	d_bandColorMap2->allocColorBands (bandBaseCol_Desert); 

	d_useLinearColormap = FALSE;

	// catch destroy event
	connect_to_method (destroy, this, &quit);

	d_win.show ();		// GTK window 
	d_win.set_policy (FALSE, FALSE, FALSE);
	d_vbox.show();		// vbox
	d_menuBarHandle->show();
	this->show();

	// since Gtk-- 0.11.0 this has to be done here in order to properly 
	// size the window when staring without loading a height field. 
	//this->set_usize (400, 400);
	this->setSize (400, 400);
	this->show ();
	}


/*
 *  destructor: clean up 
 */
TFWindow::~TFWindow()
	{
	delete d_menuHash;
	delete d_linearColorMap;
	delete d_bandColorMap;
	delete [] d_mNames;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFWindow\n");
	}


void TFWindow::quit ()
	{
	gtk_exit (0);
	}


void TFWindow::fileMenuCallback(string m)
{
	if (m == "Exit")
		quit ();
	cout << "Callback " << m << " caught " << endl;
}


void TFWindow::hfMenuCallback(string m)
{
	cout << "Callback " << m << " caught " << endl;
}


void TFWindow::viewMenuCallback(string m)
{
	cout << "Callback " << m << " caught " << endl;
}


void TFWindow::colormapMenuCallback(string m)
{
	cout << "Callback " << m << " caught " << endl;
}



void TFWindow::setSize (int x, int y)
{
	if ((this->width() != x) || (this->height() != y))
		{
		d_win.set_usize (x, y+d_menuBarHandle->height());
		this->setUsize (x, y);

		d_win.show ();
		this->show();
		gdk_flush ();

		Gtk_Main *gtkMain = Gtk_Main::instance();
		while (gtkMain->events_pending())
			gtkMain->iteration ();
		}
}


guint TFWhash (gconstpointer key)
{
	char 	*s = (char *)key;
	int	i, v=0,
		l=strlen(s);

	for (i=0; i<l; i++)
		v = (64*v + s[i])%101;

	return (v);
}


gint TFWhashcmp (gconstpointer a, gconstpointer b)
{
	return (!strcmp((char *)a,(char *)b));
}


